from enum import IntEnum
from typing_extensions import Literal
from typing import Union, Optional, Annotated

from pydantic import Field, BaseModel
from nonebot.compat import PYDANTIC_V2, ConfigDict

PAYLOAD_FIELD_ALIASES = {"opcode": "op", "data": "d", "sequence": "s", "type": "t"}


class Opcode(IntEnum):
    DISPATCH = 0
    HEARTBEAT = 1
    IDENTIFY = 2
    RESUME = 6
    RECONNECT = 7
    INVALID_SESSION = 9
    HELLO = 10
    HEARTBEAT_ACK = 11
    HTTP_CALLBACK_ACK = 12


class Payload(BaseModel):
    if PYDANTIC_V2:
        model_config: ConfigDict = ConfigDict(
            extra="allow",
            populate_by_name=True,
            alias_generator=lambda x: PAYLOAD_FIELD_ALIASES.get(x, x),
        )
    else:

        class Config:
            extra = "allow"
            allow_population_by_field_name = True

            @classmethod
            def alias_generator(cls, string: str) -> str:
                return PAYLOAD_FIELD_ALIASES.get(string, string)


class Dispatch(Payload):
    opcode: Literal[Opcode.DISPATCH] = Field(Opcode.DISPATCH)
    data: dict
    sequence: int
    type: str
    id: Optional[str] = None


class Heartbeat(Payload):
    opcode: Literal[Opcode.HEARTBEAT] = Field(Opcode.HEARTBEAT)
    data: int


class IdentifyData(BaseModel):
    token: str
    intents: int
    shard: tuple[int, int]
    properties: dict

    if PYDANTIC_V2:
        model_config: ConfigDict = ConfigDict(extra="allow")
    else:

        class Config:
            extra = "allow"


class Identify(Payload):
    opcode: Literal[Opcode.IDENTIFY] = Field(Opcode.IDENTIFY)
    data: IdentifyData


class ResumeData(BaseModel):
    token: str
    session_id: str
    seq: int

    if PYDANTIC_V2:
        model_config: ConfigDict = ConfigDict(extra="allow")
    else:

        class Config:
            extra = "allow"


class Resume(Payload):
    opcode: Literal[Opcode.RESUME] = Field(Opcode.RESUME)
    data: ResumeData


class Reconnect(Payload):
    opcode: Literal[Opcode.RECONNECT] = Field(Opcode.RECONNECT)


class InvalidSession(Payload):
    opcode: Literal[Opcode.INVALID_SESSION] = Field(Opcode.INVALID_SESSION)


class HelloData(BaseModel):
    heartbeat_interval: int

    if PYDANTIC_V2:
        model_config: ConfigDict = ConfigDict(extra="allow")
    else:

        class Config:
            extra = "allow"


class Hello(Payload):
    opcode: Literal[Opcode.HELLO] = Field(Opcode.HELLO)
    data: HelloData


class HeartbeatAck(Payload):
    opcode: Literal[Opcode.HEARTBEAT_ACK] = Field(Opcode.HEARTBEAT_ACK)


class HTTPCallbackAck(Payload):
    opcode: Literal[Opcode.HTTP_CALLBACK_ACK] = Field(Opcode.HTTP_CALLBACK_ACK)
    data: int


PayloadType = Union[
    Annotated[
        Union[Dispatch, Reconnect, InvalidSession, Hello, HeartbeatAck],
        Field(discriminator="opcode"),
    ],
    Payload,
]



import sys
import asyncio
from typing_extensions import override
from typing import Any, Literal, Optional

from nonebot.utils import escape_tag
from nonebot.exception import WebSocketClosed
from nonebot.compat import PYDANTIC_V2, type_validate_json, type_validate_python
from nonebot.drivers import (
    URL,
    Driver,
    Request,
    WebSocket,
    HTTPClientMixin,
    WebSocketClientMixin,
)

from nonebot.adapters import Adapter as BaseAdapter

from .bot import Bot
from .utils import API, log
from .store import audit_result
from .config import Config, BotInfo
from .exception import ApiNotAvailable
from .event import EVENT_CLASSES, Event, ReadyEvent, MessageAuditEvent
from .models import (
    Hello,
    Resume,
    Payload,
    Dispatch,
    Identify,
    Heartbeat,
    Reconnect,
    PayloadType,
    HeartbeatAck,
    InvalidSession,
)

RECONNECT_INTERVAL = 3.0


class Adapter(BaseAdapter):
    @override
    def __init__(self, driver: Driver, **kwargs: Any):
        super().__init__(driver, **kwargs)

        self.qq_config: Config = Config(**self.config.dict())

        self.tasks: list["asyncio.Task"] = []
        self.setup()

    @classmethod
    @override
    def get_name(cls) -> str:
        return "QQ"

    def setup(self) -> None:
        if not isinstance(self.driver, HTTPClientMixin):
            raise RuntimeError(
                f"Current driver {self.config.driver} does not support "
                "http client requests! "
                "QQ Adapter need a HTTPClient Driver to work."
            )
        if not isinstance(self.driver, WebSocketClientMixin):
            raise RuntimeError(
                f"Current driver {self.config.driver} does not support "
                "websocket client! "
                "QQ Adapter need a WebSocketClient Driver to work."
            )
        self.on_ready(self.startup)
        self.driver.on_shutdown(self.shutdown)

    async def startup(self) -> None:
        log(
            "DEBUG",
            ("QQ run in sandbox mode: " f"<y>{self.qq_config.qq_is_sandbox}</y>"),
        )

        try:
            api_base = self.get_api_base()
        except Exception as e:
            log("ERROR", "Failed to parse QQ api base url", e)
            raise

        log("DEBUG", f"QQ api base url: <y>{escape_tag(str(api_base))}</y>")

        for bot in self.qq_config.qq_bots:
            self.tasks.append(asyncio.create_task(self.run_bot(bot)))

    async def shutdown(self) -> None:
        for task in self.tasks:
            if not task.done():
                task.cancel()

        await asyncio.gather(
            *(asyncio.wait_for(task, timeout=10) for task in self.tasks),
            return_exceptions=True,
        )

    async def run_bot(self, bot_info: BotInfo) -> None:
        bot = Bot(self, bot_info.id, bot_info)

        # get sharded gateway url
        try:
            gateway_info = await bot.shard_url_get()
            ws_url = URL(gateway_info.url)
        except Exception as e:
            log(
                "ERROR",
                "<r><bg #f8bbd0>Failed to get gateway info.</bg #f8bbd0></r>",
                e,
            )
            return

        if (remain := gateway_info.session_start_limit.remaining) and remain <= 0:
            log(
                "ERROR",
                "<r><bg #f8bbd0>Failed to establish connection to QQ "
                "because of session start limit.</bg #f8bbd0></r>\n"
                f"{escape_tag(repr(gateway_info))}",
            )
            return

        # start connection in single shard mode
        if bot_info.shard is not None:
            self.tasks.append(
                asyncio.create_task(self._forward_ws(bot, ws_url, bot_info.shard))
            )
            return

        # start connection in sharding mode
        shards = gateway_info.shards or 1
        for i in range(shards):
            self.tasks.append(
                asyncio.create_task(self._forward_ws(bot, ws_url, (i, shards)))
            )
            # wait for session start concurrency limit
            await asyncio.sleep(gateway_info.session_start_limit.max_concurrency or 1)

    async def _forward_ws(self, bot: Bot, ws_url: URL, shard: tuple[int, int]) -> None:
        # ws setup request
        request = Request(
            "GET",
            ws_url,
            timeout=30.0,
        )

        heartbeat_task: Optional["asyncio.Task"] = None

        while True:
            try:
                async with self.websocket(request) as ws:
                    log(
                        "DEBUG",
                        (
                            "WebSocket Connection to "
                            f"{escape_tag(str(ws_url))} established"
                        ),
                    )

                    try:
                        # hello
                        heartbeat_interval = await self._hello(bot, ws)
                        if heartbeat_interval is None:
                            await asyncio.sleep(RECONNECT_INTERVAL)
                            continue

                        # identify/resume
                        result = await self._authenticate(bot, ws, shard)
                        if not result:
                            await asyncio.sleep(RECONNECT_INTERVAL)
                            continue

                        # start heartbeat
                        heartbeat_task = asyncio.create_task(
                            self._heartbeat(bot, ws, heartbeat_interval)
                        )

                        # process events
                        await self._loop(bot, ws)
                    except WebSocketClosed as e:
                        log(
                            "ERROR",
                            "<r><bg #f8bbd0>WebSocket Closed</bg #f8bbd0></r>",
                            e,
                        )
                    except Exception as e:
                        log(
                            "ERROR",
                            (
                                "<r><bg #f8bbd0>"
                                "Error while process data from websocket "
                                f"{escape_tag(str(ws_url))}. Trying to reconnect..."
                                "</bg #f8bbd0></r>"
                            ),
                            e,
                        )
                    finally:
                        if heartbeat_task:
                            heartbeat_task.cancel()
                            heartbeat_task = None
                        if bot.self_id in self.bots:
                            self.bot_disconnect(bot)

            except Exception as e:
                log(
                    "ERROR",
                    (
                        "<r><bg #f8bbd0>"
                        "Error while setup websocket to "
                        f"{escape_tag(str(ws_url))}. Trying to reconnect..."
                        "</bg #f8bbd0></r>"
                    ),
                    e,
                )

            await asyncio.sleep(RECONNECT_INTERVAL)

    async def _hello(self, bot: Bot, ws: WebSocket) -> Optional[int]:
        """接收并处理服务器的 Hello 事件"""
        try:
            payload = await self.receive_payload(bot, ws)
            assert isinstance(
                payload, Hello
            ), f"Received unexpected payload: {payload!r}"
            return payload.data.heartbeat_interval
        except Exception as e:
            log(
                "ERROR",
                (
                    "<r><bg #f8bbd0>"
                    "Error while receiving server hello event"
                    "</bg #f8bbd0></r>"
                ),
                e,
            )

    async def _authenticate(
        self, bot: Bot, ws: WebSocket, shard: tuple[int, int]
    ) -> Optional[Literal[True]]:
        """鉴权连接"""
        if not bot.ready:
            payload = type_validate_python(
                Identify,
                {
                    "data": {
                        "token": await bot._get_authorization_header(),
                        "intents": bot.bot_info.intent.to_int(),
                        "shard": shard,
                        "properties": {
                            "$os": sys.platform,
                            "$language": f"python {sys.version}",
                            "$sdk": "NoneBot2",
                        },
                    }
                },
            )
        else:
            payload = type_validate_python(
                Resume,
                {
                    "data": {
                        "token": await bot._get_authorization_header(),
                        "session_id": bot.session_id,
                        "seq": bot.sequence,
                    }
                },
            )

        try:
            await ws.send(self.payload_to_json(payload))
        except Exception as e:
            log(
                "ERROR",
                "<r><bg #f8bbd0>Error while sending "
                + ("Identify" if isinstance(payload, Identify) else "Resume")
                + " event</bg #f8bbd0></r>",
                e,
            )
            return

        ready_event = None
        if not bot.ready:
            # https://bot.q.qq.com/wiki/develop/api/gateway/reference.html#_2-%E9%89%B4%E6%9D%83%E8%BF%9E%E6%8E%A5
            # 鉴权成功之后，后台会下发一个 Ready Event
            payload = await self.receive_payload(bot, ws)
            if isinstance(payload, InvalidSession):
                log(
                    "WARNING",
                    "Received invalid session event from server. Try to reconnect...",
                )
                return
            elif not isinstance(payload, Dispatch):
                log(
                    "ERROR",
                    "Received unexpected payload while authenticating: "
                    f"{escape_tag(repr(payload))}",
                )
                return

            ready_event = self.payload_to_event(payload)
            if not isinstance(ready_event, ReadyEvent):
                log(
                    "ERROR",
                    "Received unexpected event while authenticating: "
                    f"{escape_tag(repr(ready_event))}",
                )
                return

            bot.on_ready(ready_event)

        # only connect for single shard
        if bot.self_id not in self.bots:
            self.bot_connect(bot)
            log(
                "INFO",
                f"<y>Bot {escape_tag(bot.self_id)}</y> connected",
            )

        if ready_event:
            asyncio.create_task(bot.handle_event(ready_event))  # noqa: RUF006

        return True

    async def _heartbeat(self, bot: Bot, ws: WebSocket, heartbeat_interval: int):
        """心跳"""
        while True:
            if bot.ready:
                log("TRACE", f"Heartbeat {bot.sequence}")
                payload = type_validate_python(Heartbeat, {"data": bot.sequence})
                try:
                    await ws.send(self.payload_to_json(payload))
                except Exception as e:
                    log("WARNING", "Error while sending heartbeat, Ignored!", e)
            await asyncio.sleep(heartbeat_interval / 1000)

    async def _loop(self, bot: Bot, ws: WebSocket):
        """接收并处理事件"""
        while True:
            payload = await self.receive_payload(bot, ws)
            log(
                "TRACE",
                f"Received payload: {escape_tag(repr(payload))}",
            )
            if isinstance(payload, Dispatch):
                try:
                    event = self.payload_to_event(payload)
                except Exception as e:
                    log(
                        "WARNING",
                        f"Failed to parse event {escape_tag(repr(payload))}",
                        e,
                    )
                else:
                    if isinstance(event, MessageAuditEvent):
                        audit_result.add_result(event)
                    asyncio.create_task(bot.handle_event(event))  # noqa: RUF006
            elif isinstance(payload, HeartbeatAck):
                log("TRACE", "Heartbeat ACK")
                continue
            elif isinstance(payload, Reconnect):
                log(
                    "WARNING",
                    "Received reconnect event from server. Try to reconnect...",
                )
                break
            elif isinstance(payload, InvalidSession):
                bot.reset()
                log(
                    "ERROR",
                    "Received invalid session event from server. Try to reconnect...",
                )
                break
            else:
                log(
                    "WARNING",
                    f"Unknown payload from server: {escape_tag(repr(payload))}",
                )

    def get_auth_base(self) -> URL:
        return URL(str(self.qq_config.qq_auth_base))

    def get_api_base(self) -> URL:
        if self.qq_config.qq_is_sandbox:
            return URL(str(self.qq_config.qq_sandbox_api_base))
        else:
            return URL(str(self.qq_config.qq_api_base))

    @staticmethod
    async def receive_payload(bot: Bot, ws: WebSocket) -> Payload:
        payload = type_validate_json(PayloadType, await ws.receive())
        if isinstance(payload, Dispatch):
            bot.on_dispatch(payload)
        return payload

    @staticmethod
    def payload_to_json(payload: Payload) -> str:
        if PYDANTIC_V2:
            return payload.model_dump_json(by_alias=True)

        return payload.json(by_alias=True)

    @staticmethod
    def payload_to_event(payload: Dispatch) -> Event:
        EventClass = EVENT_CLASSES.get(payload.type, None)
        if EventClass is None:
            log("WARNING", f"Unknown payload type: {payload.type}")
            event = type_validate_python(
                Event, {"event_id": payload.id, **payload.data}
            )
            event.__type__ = payload.type  # type: ignore
            return event
        return type_validate_python(
            EventClass, {"event_id": payload.id, **payload.data}
        )

    @override
    async def _call_api(self, bot: Bot, api: str, **data: Any) -> Any:
        log("DEBUG", f"Bot {bot.bot_info.id} calling API <y>{api}</y>")
        api_handler: Optional[API] = getattr(bot.__class__, api, None)
        if api_handler is None:
            raise ApiNotAvailable
        return await api_handler(bot, **data)



from typing import Optional

from pydantic import Field, HttpUrl, BaseModel
from nonebot.compat import PYDANTIC_V2, ConfigDict


class Intents(BaseModel):
    guilds: bool = True
    guild_members: bool = True
    guild_messages: bool = False
    guild_message_reactions: bool = True
    direct_message: bool = False
    open_forum_event: bool = False
    audio_live_member: bool = False
    c2c_group_at_messages: bool = False
    interaction: bool = False
    message_audit: bool = True
    forum_event: bool = False
    audio_action: bool = False
    at_messages: bool = True

    if PYDANTIC_V2:
        model_config: ConfigDict = ConfigDict(extra="forbid")
    else:

        class Config:
            extra = "forbid"

    def to_int(self):
        return (
            self.guilds << 0
            | self.guild_members << 1
            | self.guild_messages << 9
            | self.guild_message_reactions << 10
            | self.direct_message << 12
            | self.open_forum_event << 18
            | self.audio_live_member << 19
            | self.c2c_group_at_messages << 25
            | self.interaction << 26
            | self.message_audit << 27
            | self.forum_event << 28
            | self.audio_action << 29
            | self.at_messages << 30
        )

    @property
    def is_group_enabled(self) -> bool:
        """是否开启群聊功能"""
        return self.c2c_group_at_messages is True


class BotInfo(BaseModel):
    id: str = Field(alias="id")
    token: str = Field(alias="token")
    secret: str = Field(alias="secret")
    shard: Optional[tuple[int, int]] = None
    intent: Intents = Field(default_factory=Intents)

    @property
    def is_group_bot(self) -> bool:
        """是否为群机器人"""
        return self.intent.is_group_enabled


class Config(BaseModel):
    qq_is_sandbox: bool = False
    qq_api_base: HttpUrl = Field("https://api.sgroup.qq.com/")
    qq_sandbox_api_base: HttpUrl = Field("https://sandbox.api.sgroup.qq.com")
    qq_auth_base: HttpUrl = Field("https://bots.qq.com/app/getAppAccessToken")
    qq_bots: list[BotInfo] = Field(default_factory=list)



import json
from base64 import b64encode
from typing_extensions import Never, override
from datetime import datetime, timezone, timedelta
from typing import (
    IO,
    TYPE_CHECKING,
    Any,
    Union,
    Literal,
    NoReturn,
    Optional,
    cast,
    overload,
)

from pydantic import BaseModel
from nonebot.message import handle_event
from nonebot.drivers import Request, Response
from nonebot.compat import type_validate_python

from nonebot.adapters import Bot as BaseBot

from .config import BotInfo
from .utils import API, log, exclude_none
from .models import Message as GuildMessage
from .message import Message, MessageSegment
from .models import DMS, User, Guild, Media, Member, Channel
from .exception import (
    ActionFailed,
    NetworkError,
    AuditException,
    ApiNotAvailable,
    RateLimitException,
    UnauthorizedException,
)
from .event import (
    Event,
    ReadyEvent,
    QQMessageEvent,
    GuildMessageEvent,
    C2CMessageCreateEvent,
    InteractionCreateEvent,
    DirectMessageCreateEvent,
    GroupAtMessageCreateEvent,
)
from .models import (
    Dispatch,
    RichText,
    Schedule,
    EmojiType,
    MessageArk,
    RemindType,
    AudioStatus,
    ChannelType,
    PinsMessage,
    PrivateType,
    AudioControl,
    MessageEmbed,
    UrlGetReturn,
    ChannelSubType,
    MessageSetting,
    GetThreadReturn,
    MessageKeyboard,
    MessageMarkdown,
    PutThreadReturn,
    SpeakPermission,
    MessageReference,
    RecommendChannel,
    ShardUrlGetReturn,
    ChannelPermissions,
    PostC2CFilesReturn,
    APIPermissionDemand,
    GetGuildRolesReturn,
    PostGuildRoleReturn,
    GetRoleMembersReturn,
    GetThreadsListReturn,
    PatchGuildRoleReturn,
    PostGroupFilesReturn,
    PostC2CMessagesReturn,
    GetReactionUsersReturn,
    PostGroupMembersReturn,
    PostGroupMessagesReturn,
    APIPermissionDemandIdentify,
    GetGuildAPIPermissionReturn,
)

if TYPE_CHECKING:
    from .adapter import Adapter


async def _check_reply(
    bot: "Bot",
    event: Union[GuildMessageEvent, QQMessageEvent],
) -> None:
    """检查消息中存在的回复，赋值 `event.reply`, `event.to_me`。

    参数:
        bot: Bot 对象
        event: MessageEvent 对象
    """
    if not isinstance(event, GuildMessageEvent) or event.message_reference is None:
        return
    try:
        event.reply = await bot.get_message_of_id(
            channel_id=event.channel_id,
            message_id=event.message_reference.message_id,
        )
        if event.reply.author.id == bot.self_info.id:
            event.to_me = True
    except Exception as e:
        log("WARNING", f"Error when getting message reply info: {e!r}", e)


def _check_at_me(
    bot: "Bot",
    event: Union[GuildMessageEvent, QQMessageEvent],
):
    if (
        isinstance(event, GuildMessageEvent)
        and event.mentions is not None
        and bot.self_info.id in {user.id for user in event.mentions}
    ):
        event.to_me = True

    def _is_at_me_seg(segment: MessageSegment) -> bool:
        return (
            segment.type == "mention_user"
            and segment.data.get("user_id") == bot.self_info.id
        )

    message = event.get_message()

    # ensure message is not empty
    if not message:
        message.append(MessageSegment.text(""))

    deleted = False
    if _is_at_me_seg(message[0]):
        message.pop(0)
        deleted = True
        event.to_me = True
        if message and message[0].type == "text":
            message[0].data["text"] = message[0].data["text"].lstrip("\xa0").lstrip()
            if not message[0].data["text"]:
                del message[0]

    if not deleted:
        # check the last segment
        i = -1
        last_msg_seg = message[i]
        if (
            last_msg_seg.type == "text"
            and not last_msg_seg.data["text"].strip()
            and len(message) >= 2
        ):
            i -= 1
            last_msg_seg = message[i]

        if _is_at_me_seg(last_msg_seg):
            deleted = True
            event.to_me = True
            del message[i:]

    if not message:
        message.append(MessageSegment.text(""))


class Bot(BaseBot):
    adapter: "Adapter"

    @override
    def __init__(self, adapter: "Adapter", self_id: str, bot_info: BotInfo):
        super().__init__(adapter, self_id)

        # Bot 配置信息
        self.bot_info: BotInfo = bot_info

        # Bot 自身信息
        self._self_info: Optional[User] = None
        # Bot 当前 session id，可用于 Resume 重连
        self._session_id: Optional[str] = None
        # Bot 当前事件序号，可用于 Resume 重连
        self._sequence: Optional[int] = None

        # 群聊机器人鉴权信息
        self._access_token: Optional[str] = None
        self._expires_in: Optional[datetime] = None

    @override
    def __getattr__(self, name: str) -> NoReturn:
        raise AttributeError(
            f'"{self.__class__.__name__}" object has no attribute "{name}"'
        )

    @property
    def self_info(self) -> User:
        """Bot 自身信息，仅当 Bot 连接鉴权完成后可用"""
        if self._self_info is None:
            raise RuntimeError(f"Bot {self.bot_info} is not connected!")
        return self._self_info

    @property
    def ready(self) -> bool:
        """Bot 是否已经准备就绪"""
        return self._session_id is not None

    @property
    def session_id(self) -> str:
        """Bot 当前的 session id"""
        if self._session_id is None:
            raise RuntimeError(f"Bot {self.self_id} is not connected!")
        return self._session_id

    @property
    def sequence(self) -> int:
        """Bot 当前事件序列号"""
        if self._sequence is None:
            raise RuntimeError(f"Bot {self.bot_info.id} is not connected!")
        return self._sequence

    def on_ready(self, event: ReadyEvent) -> None:
        self._self_info = event.user
        self._session_id = event.session_id

    def on_dispatch(self, payload: Dispatch) -> None:
        self._sequence = payload.sequence

    def reset(self) -> None:
        """清理 Bot 连接信息"""
        self._session_id = None
        self._sequence = None

    async def get_access_token(self) -> str:
        if self._access_token is None or (
            self._expires_in
            and datetime.now(timezone.utc) > self._expires_in - timedelta(seconds=30)
        ):
            request = Request(
                "POST",
                self.adapter.get_auth_base(),
                json={
                    "appId": self.bot_info.id,
                    "clientSecret": self.bot_info.secret,
                },
            )
            resp = await self.adapter.request(request)
            if resp.status_code != 200 or not resp.content:
                raise NetworkError(
                    f"Get authorization failed with status code {resp.status_code}."
                    " Please check your config."
                )
            data = json.loads(resp.content)
            self._access_token = cast(str, data["access_token"])
            self._expires_in = datetime.now(timezone.utc) + timedelta(
                seconds=int(data["expires_in"])
            )
        return self._access_token

    async def _get_authorization_header(self) -> str:
        """获取当前 Bot 的鉴权信息"""
        if self.bot_info.is_group_bot:
            return f"QQBot {await self.get_access_token()}"
        return f"Bot {self.bot_info.id}.{self.bot_info.token}"

    async def get_authorization_header(self) -> dict[str, str]:
        """获取当前 Bot 的鉴权信息"""
        headers = {"Authorization": await self._get_authorization_header()}
        if self.bot_info.is_group_bot:
            headers["X-Union-Appid"] = self.bot_info.id
        return headers

    async def handle_event(self, event: Event) -> None:
        if isinstance(event, (GuildMessageEvent, QQMessageEvent)):
            await _check_reply(self, event)
            _check_at_me(self, event)
        await handle_event(self, event)

    @staticmethod
    def _prepare_message(message: Union[str, Message, MessageSegment]) -> Message:
        _message = MessageSegment.text(message) if isinstance(message, str) else message
        _message = _message if isinstance(_message, Message) else Message(_message)
        return _message

    @staticmethod
    def _extract_send_message(message: Message) -> dict[str, Any]:
        kwargs = {}
        content = message.extract_content() or None
        kwargs["content"] = content
        if embed := (message["embed"] or None):
            kwargs["embed"] = embed[-1].data["embed"]
        if ark := (message["ark"] or None):
            kwargs["ark"] = ark[-1].data["ark"]
        if markdown := (message["markdown"] or None):
            kwargs["markdown"] = markdown[-1].data["markdown"]
        if reference := (message["reference"] or None):
            kwargs["message_reference"] = reference[-1].data["reference"]
        if keyboard := (message["keyboard"] or None):
            kwargs["keyboard"] = keyboard[-1].data["keyboard"]
        return kwargs

    @staticmethod
    def _extract_guild_image(message: Message) -> dict[str, Any]:
        kwargs = {}
        if image := (message["image"] or None):
            kwargs["image"] = image[-1].data["url"]
        if file_image := (message["file_image"] or None):
            kwargs["file_image"] = file_image[-1].data["content"]
        return kwargs

    @staticmethod
    def _extract_qq_media(message: Message) -> dict[str, Any]:
        kwargs = {}
        if image := message["image"]:
            kwargs["file_type"] = 1
            kwargs["url"] = image[-1].data["url"]
        elif video := message["video"]:
            kwargs["file_type"] = 2
            kwargs["url"] = video[-1].data["url"]
        elif audio := message["audio"]:
            kwargs["file_type"] = 3
            kwargs["url"] = audio[-1].data["url"]
        elif file := message["file"]:
            kwargs["file_type"] = 4
            kwargs["url"] = file[-1].data["url"]
        elif file_image := message["file_image"]:
            kwargs["file_type"] = 1
            kwargs["file_data"] = file_image[-1].data["content"]
        elif file_video := message["file_video"]:
            kwargs["file_type"] = 2
            kwargs["file_data"] = file_video[-1].data["content"]
        elif file_audio := message["file_audio"]:
            kwargs["file_type"] = 3
            kwargs["file_data"] = file_audio[-1].data["content"]
        elif file_file := message["file_file"]:
            kwargs["file_type"] = 4
            kwargs["file_data"] = file_file[-1].data["content"]
        return kwargs

    async def send_to_dms(
        self,
        guild_id: str,
        message: Union[str, Message, MessageSegment],
        msg_id: Optional[str] = None,
        event_id: Optional[str] = None,
    ) -> GuildMessage:
        message = self._prepare_message(message)
        return await self.post_dms_messages(
            guild_id=guild_id,
            msg_id=msg_id,
            event_id=event_id,
            **self._extract_send_message(message=message),
            **self._extract_guild_image(message=message),
        )

    async def send_to_channel(
        self,
        channel_id: str,
        message: Union[str, Message, MessageSegment],
        msg_id: Optional[str] = None,
        event_id: Optional[str] = None,
    ) -> GuildMessage:
        message = self._prepare_message(message)
        return await self.post_messages(
            channel_id=channel_id,
            msg_id=msg_id,
            event_id=event_id,
            **self._extract_send_message(message=message),
            **self._extract_guild_image(message=message),
        )

    async def send_to_c2c(
        self,
        openid: str,
        message: Union[str, Message, MessageSegment],
        msg_id: Optional[str] = None,
        msg_seq: Optional[int] = None,
        event_id: Optional[str] = None,
    ) -> Union[PostC2CMessagesReturn, PostC2CFilesReturn]:
        message = self._prepare_message(message)
        kwargs = self._extract_send_message(message=message)
        if kwargs.get("embed"):
            msg_type = 4
        elif kwargs.get("ark"):
            msg_type = 3
        elif kwargs.get("markdown"):
            msg_type = 2
        elif (
            message["image"]
            or message["audio"]
            or message["video"]
            or message["file"]
            or message["file_image"]
            or message["file_audio"]
            or message["file_video"]
            or message["file_file"]
        ):
            msg_type = 7
        else:
            msg_type = 0

        media: Optional[Media] = None
        if msg_type == 7:
            media_info = await self.post_c2c_files(
                openid=openid, srv_send_msg=False, **self._extract_qq_media(message)
            )
            media = (
                Media(file_info=media_info.file_info) if media_info.file_info else None
            )
        kwargs["media"] = media

        return await self.post_c2c_messages(
            openid=openid,
            msg_type=msg_type,
            msg_id=msg_id,
            msg_seq=msg_seq,
            event_id=event_id,
            **kwargs,
        )

    async def send_to_group(
        self,
        group_openid: str,
        message: Union[str, Message, MessageSegment],
        msg_id: Optional[str] = None,
        msg_seq: Optional[int] = None,
        event_id: Optional[str] = None,
    ) -> Union[PostGroupMessagesReturn, PostGroupFilesReturn]:
        message = self._prepare_message(message)
        kwargs = self._extract_send_message(message=message)
        if kwargs.get("embed"):
            msg_type = 4
        elif kwargs.get("ark"):
            msg_type = 3
        elif kwargs.get("markdown") or kwargs.get("keyboard"):
            msg_type = 2
        elif (
            message["image"]
            or message["audio"]
            or message["video"]
            or message["file"]
            or message["file_image"]
            or message["file_audio"]
            or message["file_video"]
            or message["file_file"]
        ):
            msg_type = 7
        else:
            msg_type = 0

        media: Optional[Media] = None
        if msg_type == 7:
            media_info = await self.post_group_files(
                group_openid=group_openid,
                srv_send_msg=False,
                **self._extract_qq_media(message),
            )
            media = (
                Media(file_info=media_info.file_info) if media_info.file_info else None
            )
        kwargs["media"] = media

        return await self.post_group_messages(
            group_openid=group_openid,
            msg_type=msg_type,
            msg_id=msg_id,
            msg_seq=msg_seq,
            event_id=event_id,
            **kwargs,
        )

    @override
    async def send(
        self,
        event: Event,
        message: Union[str, Message, MessageSegment],
        **kwargs,
    ) -> Any:
        if isinstance(event, DirectMessageCreateEvent):
            # 私信需要使用 post_dms_messages
            # https://bot.q.qq.com/wiki/develop/api/openapi/dms/post_dms_messages.html#%E5%8F%91%E9%80%81%E7%A7%81%E4%BF%A1
            return await self.send_to_dms(
                guild_id=event.guild_id,
                message=message,
                msg_id=event.id,
            )
        elif isinstance(event, GuildMessageEvent):
            return await self.send_to_channel(
                channel_id=event.channel_id,
                message=message,
                msg_id=event.id,
            )
        elif isinstance(event, C2CMessageCreateEvent):
            event._reply_seq += 1
            return await self.send_to_c2c(
                openid=event.author.id,
                message=message,
                msg_id=event.id,
                msg_seq=event._reply_seq,
            )
        elif isinstance(event, GroupAtMessageCreateEvent):
            event._reply_seq += 1
            return await self.send_to_group(
                group_openid=event.group_openid,
                message=message,
                msg_id=event.id,
                msg_seq=event._reply_seq,
            )
        elif isinstance(event, InteractionCreateEvent):
            if gid := event.group_openid:
                return await self.send_to_group(
                    group_openid=gid, event_id=event.event_id, message=message
                )
            elif cid := event.channel_id:
                return await self.send_to_channel(
                    channel_id=cid, event_id=event.event_id, message=message
                )
            elif uid := event.user_openid:
                return await self.send_to_c2c(
                    openid=uid, event_id=event.event_id, message=message
                )
            elif gid := event.guild_id:
                return await self.send_to_dms(
                    guild_id=gid, event_id=event.event_id, message=message
                )

        raise RuntimeError("Event cannot be replied to!")

    # API request methods
    def _handle_response(self, response: Response) -> Any:
        if trace_id := response.headers.get("X-Tps-trace-ID", None):
            log(
                "TRACE",
                f"Called API {response.request and response.request.url} "
                f"response {response.status_code} with trace id {trace_id}",
            )
        if response.status_code == 201 or response.status_code == 202:
            if response.content and (content := json.loads(response.content)):
                audit_id = (
                    content.get("data", {})
                    .get("message_audit", {})
                    .get("audit_id", None)
                )
                if audit_id:
                    raise AuditException(audit_id)
            raise ActionFailed(response)
        elif 200 <= response.status_code < 300:
            return response.content and json.loads(response.content)
        elif response.status_code == 401:
            raise UnauthorizedException(response)
        elif response.status_code in (404, 405):
            raise ApiNotAvailable
        elif response.status_code == 429:
            raise RateLimitException(response)
        else:
            raise ActionFailed(response)

    async def _request(self, request: Request) -> Any:
        request.headers.update(await self.get_authorization_header())

        try:
            response = await self.adapter.request(request)
        except Exception as e:
            raise NetworkError("API request failed") from e

        try:
            return self._handle_response(response)
        except UnauthorizedException as e:
            if not self.bot_info.is_group_bot:
                raise

            log("DEBUG", "Access token expired, try to refresh it.")

            # try to refresh access token
            self._access_token = None
            try:
                request.headers.update(await self.get_authorization_header())
            except Exception:
                raise e from None

            # resend request
            try:
                response = await self.adapter.request(request)
            except Exception as ex:
                raise NetworkError("API request failed") from ex

            try:
                return self._handle_response(response)
            except Exception as ex:
                raise ex from None

    # User API
    @API
    async def me(self) -> User:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("users/@me"),
        )
        return type_validate_python(User, await self._request(request))

    @API
    async def guilds(
        self,
        *,
        before: Optional[str] = None,
        after: Optional[str] = None,
        limit: Optional[float] = None,
    ) -> list[Guild]:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("users", "@me", "guilds"),
            params=exclude_none({"before": before, "after": after, "limit": limit}),
        )
        return type_validate_python(list[Guild], await self._request(request))

    # Guild API
    @API
    async def get_guild(self, *, guild_id: str) -> Guild:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("guilds", guild_id),
        )
        return type_validate_python(Guild, await self._request(request))

    # Channel API
    @API
    async def get_channels(self, *, guild_id: str) -> list[Channel]:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("guilds", guild_id, "channels"),
        )
        return type_validate_python(list[Channel], await self._request(request))

    @API
    async def get_channel(self, *, channel_id: str) -> Channel:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("channels", channel_id),
        )
        return type_validate_python(Channel, await self._request(request))

    @API
    async def post_channels(
        self,
        *,
        guild_id: str,
        name: str,
        type: Union[ChannelType, int],
        sub_type: Union[ChannelSubType, int],
        position: Optional[int] = None,
        parent_id: Optional[int] = None,
        private_type: Optional[Union[PrivateType, int]] = None,
        private_user_ids: Optional[list[str]] = None,
        speak_permission: Optional[Union[SpeakPermission, int]] = None,
        application_id: Optional[str] = None,
    ) -> list[Channel]:
        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("guilds", guild_id, "channels"),
            json=exclude_none(
                {
                    "name": name,
                    "type": type,
                    "sub_type": sub_type,
                    "position": position,
                    "parent_id": parent_id,
                    "private_type": private_type,
                    "private_user_ids": private_user_ids,
                    "speaking_permission": speak_permission,
                    "application_id": application_id,
                }
            ),
        )
        return type_validate_python(list[Channel], await self._request(request))

    @API
    async def patch_channel(
        self,
        *,
        channel_id: str,
        name: Optional[str] = None,
        type: Optional[Union[ChannelType, int]] = None,
        sub_type: Optional[Union[ChannelSubType, int]] = None,
        position: Optional[int] = None,
        parent_id: Optional[int] = None,
        private_type: Optional[int] = None,
        speak_permission: Optional[Union[SpeakPermission, int]] = None,
        application_id: Optional[str] = None,
    ) -> Channel:
        request = Request(
            "PATCH",
            self.adapter.get_api_base().joinpath("channels", channel_id),
            json=exclude_none(
                {
                    "name": name,
                    "type": type,
                    "sub_type": sub_type,
                    "position": position,
                    "parent_id": parent_id,
                    "private_type": private_type,
                    "speaking_permission": speak_permission,
                    "application_id": application_id,
                }
            ),
        )
        return type_validate_python(Channel, await self._request(request))

    @API
    async def delete_channel(self, *, channel_id: str) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath("channels", channel_id),
        )
        return await self._request(request)

    # Member API
    @API
    async def get_members(
        self,
        *,
        guild_id: str,
        after: Optional[str] = None,
        limit: Optional[float] = None,
    ) -> list[Member]:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("guilds", guild_id, "members"),
            params=exclude_none({"after": after, "limit": limit}),
        )
        return type_validate_python(list[Member], await self._request(request))

    @API
    async def get_role_members(
        self,
        *,
        guild_id: str,
        role_id: str,
        start_index: Optional[str] = None,
        limit: Optional[int] = None,
    ) -> GetRoleMembersReturn:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath(
                "guilds", guild_id, "roles", role_id, "members"
            ),
            params=exclude_none({"start_index": start_index, "limit": limit}),
        )
        return type_validate_python(GetRoleMembersReturn, await self._request(request))

    @API
    async def get_member(self, *, guild_id: str, user_id: str) -> Member:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath(
                "guilds", guild_id, "members", user_id
            ),
        )
        return type_validate_python(Member, await self._request(request))

    @API
    async def delete_member(
        self,
        *,
        guild_id: str,
        user_id: str,
        add_blacklist: Optional[bool] = None,
        delete_history_msg_days: Optional[Literal[-1, 0, 3, 7, 15, 30]] = None,
    ) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath(
                "guilds", guild_id, "members", user_id
            ),
            json=exclude_none(
                {
                    "add_blacklist": add_blacklist,
                    "delete_history_msg_days": delete_history_msg_days,
                }
            ),
        )
        return await self._request(request)

    # Role API
    @API
    async def get_guild_roles(self, *, guild_id: str) -> GetGuildRolesReturn:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("guilds", guild_id, "roles"),
        )
        return type_validate_python(GetGuildRolesReturn, await self._request(request))

    @API
    async def post_guild_role(
        self,
        *,
        guild_id: str,
        name: Optional[str] = None,
        color: Optional[float] = None,
        hoist: Optional[bool] = None,
    ) -> PostGuildRoleReturn:
        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("guilds", guild_id, "roles"),
            json=exclude_none(
                {
                    "name": name,
                    "color": color,
                    "hoist": int(hoist) if hoist is not None else None,
                }
            ),
        )
        return type_validate_python(PostGuildRoleReturn, await self._request(request))

    @API
    async def patch_guild_role(
        self,
        *,
        guild_id: str,
        role_id: str,
        name: Optional[str] = None,
        color: Optional[float] = None,
        hoist: Optional[bool] = None,
    ) -> PatchGuildRoleReturn:
        request = Request(
            "PATCH",
            self.adapter.get_api_base().joinpath("guilds", guild_id, "roles", role_id),
            json=exclude_none(
                {
                    "name": name,
                    "color": color,
                    "hoist": int(hoist) if hoist is not None else None,
                }
            ),
        )
        return type_validate_python(PatchGuildRoleReturn, await self._request(request))

    @API
    async def delete_guild_role(self, *, guild_id: str, role_id: str) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath("guilds", guild_id, "roles", role_id),
        )
        return await self._request(request)

    @API
    async def put_guild_member_role(
        self,
        *,
        guild_id: str,
        role_id: str,
        user_id: str,
        channel_id: Optional[str] = None,
    ) -> None:
        request = Request(
            "PUT",
            self.adapter.get_api_base().joinpath(
                "guilds", guild_id, "members", user_id, "roles", role_id
            ),
            json={"channel": {"id": channel_id}} if channel_id is not None else None,
        )
        return await self._request(request)

    @API
    async def delete_guild_member_role(
        self,
        *,
        guild_id: str,
        role_id: str,
        user_id: str,
        channel_id: Optional[str] = None,
    ) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath(
                "guilds", guild_id, "members", user_id, "roles", role_id
            ),
            json={"channel": {"id": channel_id}} if channel_id is not None else None,
        )
        return await self._request(request)

    # Permission API
    @API
    async def get_channel_permissions(
        self, *, channel_id: str, user_id: str
    ) -> ChannelPermissions:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "members", user_id, "permissions"
            ),
        )
        return type_validate_python(ChannelPermissions, await self._request(request))

    @API
    async def put_channel_permissions(
        self,
        *,
        channel_id: str,
        user_id: str,
        add: Optional[int] = None,
        remove: Optional[int] = None,
    ) -> None:
        request = Request(
            "PUT",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "members", user_id, "permissions"
            ),
            json=exclude_none({"add": str(add), "remove": str(remove)}),
        )
        return await self._request(request)

    @API
    async def get_channel_roles_permissions(
        self, *, channel_id: str, role_id: str
    ) -> ChannelPermissions:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "roles", role_id, "permissions"
            ),
        )
        return type_validate_python(ChannelPermissions, await self._request(request))

    @API
    async def put_channel_roles_permissions(
        self,
        *,
        channel_id: str,
        role_id: str,
        add: Optional[int] = None,
        remove: Optional[int] = None,
    ) -> None:
        request = Request(
            "PUT",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "roles", role_id, "permissions"
            ),
            json=exclude_none({"add": str(add), "remove": str(remove)}),
        )
        return await self._request(request)

    # Message API
    @API
    async def get_message_of_id(
        self, *, channel_id: str, message_id: str
    ) -> GuildMessage:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "messages", message_id
            ),
        )
        result = await self._request(request)
        if isinstance(result, dict) and "message" in result:
            result = result["message"]
        return type_validate_python(GuildMessage, result)

    @staticmethod
    def _parse_send_message(data: dict[str, Any]) -> dict[str, Any]:
        data = exclude_none(data)
        data = {
            k: v.dict(exclude_none=True) if isinstance(v, BaseModel) else v
            for k, v in data.items()
        }
        if file_image := data.pop("file_image", None):
            # 使用 multipart/form-data
            multipart_files: dict[str, Any] = {"file_image": ("file_image", file_image)}
            multipart_data: dict[str, Any] = {}
            for k, v in data.items():
                if isinstance(v, (dict, list)):
                    # 当字段类型为对象或数组时需要将字段序列化为 JSON 字符串后进行调用
                    # https://bot.q.qq.com/wiki/develop/api/openapi/message/post_messages.html#content-type
                    multipart_files[k] = (
                        k,
                        json.dumps({k: v}).encode("utf-8"),
                        "application/json",
                    )
                else:
                    multipart_data[k] = v
            params = {"files": multipart_files, "data": multipart_data}
        else:
            params = {"json": data}
        return params

    @API
    async def post_messages(
        self,
        *,
        channel_id: str,
        content: Optional[str] = None,
        embed: Optional[MessageEmbed] = None,
        ark: Optional[MessageArk] = None,
        message_reference: Optional[MessageReference] = None,
        image: Optional[str] = None,
        file_image: Optional[Union[bytes, IO[bytes]]] = None,
        markdown: Optional[MessageMarkdown] = None,
        msg_id: Optional[str] = None,
        event_id: Optional[str] = None,
        keyboard: Optional[MessageKeyboard] = None,
    ) -> GuildMessage:
        params = self._parse_send_message(
            {
                "content": content,
                "embed": embed,
                "ark": ark,
                "message_reference": message_reference,
                "image": image,
                "file_image": file_image,
                "markdown": markdown,
                "msg_id": msg_id,
                "event_id": event_id,
                "keyboard": keyboard,
            }
        )
        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("channels", channel_id, "messages"),
            **params,
        )
        return type_validate_python(GuildMessage, await self._request(request))

    @API
    async def delete_message(
        self,
        *,
        channel_id: str,
        message_id: str,
        hidetip: Optional[bool] = None,
    ) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "messages", message_id
            ),
            params={"hidetip": str(hidetip).lower()} if hidetip is not None else None,
        )
        return await self._request(request)

    # Message Setting API
    @API
    async def get_message_setting(self, *, guild_id: str) -> MessageSetting:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath(
                "guilds", guild_id, "message", "setting"
            ),
        )
        return type_validate_python(MessageSetting, await self._request(request))

    # DMS API
    @API
    async def post_dms(self, *, recipient_id: str, source_guild_id: str) -> DMS:
        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("users", "@me", "dms"),
            json=exclude_none(
                {"recipient_id": recipient_id, "source_guild_id": source_guild_id}
            ),
        )
        return type_validate_python(DMS, await self._request(request))

    @API
    async def post_dms_messages(
        self,
        *,
        guild_id: str,
        content: Optional[str] = None,
        embed: Optional[MessageEmbed] = None,
        ark: Optional[MessageArk] = None,
        message_reference: Optional[MessageReference] = None,
        image: Optional[str] = None,
        file_image: Optional[Union[bytes, IO[bytes]]] = None,
        markdown: Optional[MessageMarkdown] = None,
        msg_id: Optional[str] = None,
        event_id: Optional[str] = None,
        keyboard: Optional[MessageKeyboard] = None,
    ) -> GuildMessage:
        params = self._parse_send_message(
            {
                "content": content,
                "embed": embed,
                "ark": ark,
                "message_reference": message_reference,
                "image": image,
                "file_image": file_image,
                "markdown": markdown,
                "msg_id": msg_id,
                "event_id": event_id,
                "keyboard": keyboard,
            }
        )
        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("dms", guild_id, "messages"),
            **params,
        )
        return type_validate_python(GuildMessage, await self._request(request))

    @API
    async def delete_dms_message(
        self, *, guild_id: str, message_id: str, hidetip: Optional[bool] = None
    ) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath(
                "dms", guild_id, "messages", message_id
            ),
            params={"hidetip": str(hidetip).lower()} if hidetip is not None else None,
        )
        return await self._request(request)

    # Mute API
    @API
    async def patch_guild_mute(
        self,
        *,
        guild_id: str,
        mute_end_timestamp: Optional[Union[int, datetime]] = None,
        mute_seconds: Optional[Union[int, timedelta]] = None,
    ) -> None:
        if isinstance(mute_end_timestamp, datetime):
            mute_end_timestamp = int(mute_end_timestamp.timestamp())

        if isinstance(mute_seconds, timedelta):
            mute_seconds = int(mute_seconds.total_seconds())

        request = Request(
            "PATCH",
            self.adapter.get_api_base().joinpath("guilds", guild_id, "mute"),
            json=exclude_none(
                {
                    "mute_end_timestamp": str(mute_end_timestamp),
                    "mute_seconds": str(mute_seconds),
                }
            ),
        )
        return await self._request(request)

    @API
    async def patch_guild_member_mute(
        self,
        *,
        guild_id: str,
        user_id: str,
        mute_end_timestamp: Optional[Union[int, datetime]] = None,
        mute_seconds: Optional[Union[int, timedelta]] = None,
    ) -> None:
        if isinstance(mute_end_timestamp, datetime):
            mute_end_timestamp = int(mute_end_timestamp.timestamp())

        if isinstance(mute_seconds, timedelta):
            mute_seconds = int(mute_seconds.total_seconds())

        request = Request(
            "PATCH",
            self.adapter.get_api_base().joinpath(
                "guilds", guild_id, "members", user_id, "mute"
            ),
            json=exclude_none(
                {
                    "mute_end_timestamp": str(mute_end_timestamp),
                    "mute_seconds": str(mute_seconds),
                }
            ),
        )
        return await self._request(request)

    @API
    async def patch_guild_mute_multi_member(
        self,
        *,
        guild_id: str,
        user_ids: list[str],
        mute_end_timestamp: Optional[Union[int, datetime]] = None,
        mute_seconds: Optional[Union[int, timedelta]] = None,
    ) -> list[int]:
        if isinstance(mute_end_timestamp, datetime):
            mute_end_timestamp = int(mute_end_timestamp.timestamp())

        if isinstance(mute_seconds, timedelta):
            mute_seconds = int(mute_seconds.total_seconds())

        request = Request(
            "PATCH",
            self.adapter.get_api_base().joinpath("guilds", guild_id, "mute"),
            json=exclude_none(
                {
                    "user_ids": user_ids,
                    "mute_end_timestamp": str(mute_end_timestamp),
                    "mute_seconds": str(mute_seconds),
                }
            ),
        )
        return type_validate_python(list[int], await self._request(request))

    # Announce API
    @API
    async def post_guild_announces(
        self,
        *,
        guild_id: str,
        message_id: Optional[str] = None,
        channel_id: Optional[str] = None,
        announces_type: Optional[int] = None,
        recommend_channels: Optional[list[RecommendChannel]] = None,
    ) -> None:
        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("guilds", guild_id, "announces"),
            json=exclude_none(
                {
                    "message_id": message_id,
                    "channel_id": channel_id,
                    "announces_type": announces_type,
                    "recommend_channels": (
                        [r.dict(exclude_none=True) for r in recommend_channels]
                        if recommend_channels is not None
                        else None
                    ),
                }
            ),
        )
        return await self._request(request)

    @API
    async def delete_guild_announces(self, *, guild_id: str, message_id: str) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath(
                "guilds", guild_id, "announces", message_id
            ),
        )
        return await self._request(request)

    # Pins API
    @API
    async def put_pins_message(
        self, *, channel_id: str, message_id: str
    ) -> PinsMessage:
        request = Request(
            "PUT",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "pins", message_id
            ),
        )
        return type_validate_python(PinsMessage, await self._request(request))

    @API
    async def delete_pins_message(self, *, channel_id: str, message_id: str) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "pins", message_id
            ),
        )
        return await self._request(request)

    @API
    async def get_pins_message(self, *, channel_id: str) -> PinsMessage:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("channels", channel_id, "pins"),
        )
        return type_validate_python(PinsMessage, await self._request(request))

    # Schedule API
    @API
    async def get_schedules(
        self, *, channel_id: str, since: Optional[Union[int, datetime]] = None
    ) -> list[Schedule]:
        if isinstance(since, datetime):
            since = int(since.timestamp() * 1000)

        request = Request(
            "GET",
            self.adapter.get_api_base() / f"channels/{channel_id}/schedules",
            json=exclude_none({"since": since}),
        )
        return type_validate_python(list[Schedule], await self._request(request))

    @API
    async def get_schedule(self, *, channel_id: str, schedule_id: str) -> Schedule:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "schedules", schedule_id
            ),
        )
        return type_validate_python(Schedule, await self._request(request))

    @API
    async def post_schedule(
        self,
        *,
        channel_id: str,
        name: str,
        description: Optional[str] = None,
        start_timestamp: Union[int, datetime],
        end_timestamp: Union[int, datetime],
        jump_channel_id: Optional[str] = None,
        remind_type: Union[RemindType, int],
    ) -> Schedule:
        if isinstance(start_timestamp, datetime):
            start_timestamp = int(start_timestamp.timestamp() * 1000)

        if isinstance(end_timestamp, datetime):
            end_timestamp = int(end_timestamp.timestamp() * 1000)

        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("channels", channel_id, "schedules"),
            json={
                "schedule": exclude_none(
                    {
                        "name": name,
                        "description": description,
                        "start_timestamp": (
                            str(start_timestamp)
                            if start_timestamp is not None
                            else None
                        ),
                        "end_timestamp": (
                            str(end_timestamp) if end_timestamp is not None else None
                        ),
                        "jump_channel_id": jump_channel_id,
                        "remind_type": str(remind_type),
                    }
                )
            },
        )
        return type_validate_python(Schedule, await self._request(request))

    @API
    async def patch_schedule(
        self,
        *,
        channel_id: str,
        schedule_id: str,
        name: Optional[str] = None,
        description: Optional[str] = None,
        start_timestamp: Optional[Union[int, datetime]] = None,
        end_timestamp: Optional[Union[int, datetime]] = None,
        jump_channel_id: Optional[int] = None,
        remind_type: Optional[Union[RemindType, int]] = None,
    ) -> Schedule:
        if isinstance(start_timestamp, datetime):
            start_timestamp = int(start_timestamp.timestamp() * 1000)

        if isinstance(end_timestamp, datetime):
            end_timestamp = int(end_timestamp.timestamp() * 1000)

        request = Request(
            "PATCH",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "schedules", schedule_id
            ),
            json={
                "schedule": exclude_none(
                    {
                        "name": name,
                        "description": description,
                        "start_timestamp": (
                            str(start_timestamp)
                            if start_timestamp is not None
                            else None
                        ),
                        "end_timestamp": (
                            str(end_timestamp) if end_timestamp is not None else None
                        ),
                        "jump_channel_id": jump_channel_id,
                        "remind_type": (
                            str(remind_type) if remind_type is not None else None
                        ),
                    }
                )
            },
        )
        return type_validate_python(Schedule, await self._request(request))

    @API
    async def delete_schedule(self, *, channel_id: str, schedule_id: str) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "schedules", schedule_id
            ),
        )
        return await self._request(request)

    @API
    async def put_message_reaction(
        self, *, channel_id: str, message_id: str, type: Union[EmojiType, int], id: str
    ) -> None:
        request = Request(
            "PUT",
            self.adapter.get_api_base().joinpath(
                "channels",
                channel_id,
                "messages",
                message_id,
                "reactions",
                str(type),
                id,
            ),
        )
        return await self._request(request)

    @API
    async def delete_own_message_reaction(
        self, *, channel_id: str, message_id: str, type: Union[EmojiType, int], id: str
    ) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath(
                "channels",
                channel_id,
                "messages",
                message_id,
                "reactions",
                str(type),
                id,
            ),
        )
        return await self._request(request)

    @API
    async def get_reaction_users(
        self,
        *,
        channel_id: str,
        message_id: str,
        type: Union[EmojiType, int],
        id: str,
        cookie: Optional[str] = None,
        limit: Optional[int] = None,
    ) -> GetReactionUsersReturn:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath(
                "channels",
                channel_id,
                "messages",
                message_id,
                "reactions",
                str(type),
                id,
            ),
            params=exclude_none({"cookie": cookie, "limit": limit}),
        )
        return await self._request(request)

    # Audio API
    @API
    async def audio_control(
        self,
        *,
        channel_id: str,
        audio_url: Optional[str] = None,
        text: Optional[str] = None,
        status: Union[AudioStatus, int],
    ) -> dict[Never, Never]:
        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("channels", channel_id, "audio"),
            json=AudioControl(audio_url=audio_url, text=text, status=status).dict(
                exclude_none=True
            ),
        )
        return await self._request(request)

    @API
    async def put_mic(self, *, channel_id: str) -> dict[Never, Never]:
        request = Request(
            "PUT",
            self.adapter.get_api_base().joinpath("channels", channel_id, "mic"),
        )
        return await self._request(request)

    @API
    async def delete_mic(self, *, channel_id: str) -> dict[Never, Never]:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath("channels", channel_id, "mic"),
        )
        return await self._request(request)

    # Forum API
    @API
    async def get_threads_list(self, *, channel_id: str) -> GetThreadsListReturn:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("channels", channel_id, "threads"),
        )
        return type_validate_python(GetThreadsListReturn, await self._request(request))

    @API
    async def get_thread(self, *, channel_id: str, thread_id: str) -> GetThreadReturn:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "threads", thread_id
            ),
        )
        return type_validate_python(GetThreadReturn, await self._request(request))

    @overload
    async def put_thread(
        self,
        *,
        channel_id: str,
        title: str,
        content: str,
        format: Literal[1, 2, 3],
    ) -> PutThreadReturn: ...

    @overload
    async def put_thread(
        self,
        *,
        channel_id: str,
        title: str,
        content: RichText,
        format: Literal[4],
    ) -> PutThreadReturn: ...

    @API
    async def put_thread(
        self,
        *,
        channel_id: str,
        title: str,
        content: Union[str, RichText],
        format: Literal[1, 2, 3, 4],
    ) -> PutThreadReturn:
        request = Request(
            "PUT",
            self.adapter.get_api_base().joinpath("channels", channel_id, "threads"),
            json=exclude_none(
                {
                    "title": title,
                    "content": (
                        content.json(exclude_none=True)
                        if isinstance(content, RichText)
                        else content
                    ),
                    "format": format,
                }
            ),
        )
        return type_validate_python(PutThreadReturn, await self._request(request))

    @API
    async def delete_thread(self, *, channel_id: str, thread_id: str) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath(
                "channels", channel_id, "threads", thread_id
            ),
        )
        return await self._request(request)

    # API Permission API
    @API
    async def get_guild_api_permission(
        self, *, guild_id: str
    ) -> GetGuildAPIPermissionReturn:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("guilds", guild_id, "api_permission"),
        )
        return type_validate_python(
            GetGuildAPIPermissionReturn, await self._request(request)
        )

    @API
    async def post_api_permission_demand(
        self,
        *,
        guild_id: str,
        channel_id: str,
        api_identify: APIPermissionDemandIdentify,
        desc: str,
    ) -> APIPermissionDemand:
        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath(
                "guilds", guild_id, "api_permission", "demand"
            ),
            json=exclude_none(
                {
                    "channel_id": channel_id,
                    "api_identify": api_identify.dict(exclude_none=True),
                    "desc": desc,
                }
            ),
        )
        return type_validate_python(APIPermissionDemand, await self._request(request))

    # WebSocket API
    @API
    async def url_get(self) -> UrlGetReturn:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("gateway"),
        )
        return type_validate_python(UrlGetReturn, await self._request(request))

    @API
    async def shard_url_get(self) -> ShardUrlGetReturn:
        request = Request(
            "GET",
            self.adapter.get_api_base().joinpath("gateway", "bot"),
        )
        return type_validate_python(ShardUrlGetReturn, await self._request(request))

    # Interaction API
    @API
    async def put_interaction(
        self, *, interaction_id: str, code: Literal[0, 1, 2, 3, 4, 5]
    ) -> None:
        request = Request(
            "PUT",
            self.adapter.get_api_base().joinpath("interactions", interaction_id),
            json={"code": code},
        )
        return await self._request(request)

    # C2C API
    @API
    async def post_c2c_messages(
        self,
        *,
        openid: str,
        msg_type: Literal[0, 1, 2, 3, 4, 7],
        content: Optional[str] = None,
        markdown: Optional[MessageMarkdown] = None,
        keyboard: Optional[MessageKeyboard] = None,
        media: Optional[Media] = None,
        ark: Optional[MessageArk] = None,
        embed: Optional[MessageEmbed] = None,
        image: None = None,
        message_reference: None = None,
        event_id: Optional[str] = None,
        msg_id: Optional[str] = None,
        msg_seq: Optional[int] = None,
        timestamp: Optional[Union[int, datetime]] = None,
    ) -> PostC2CMessagesReturn:
        # tmp fix. content must not be none if sending media
        if media is not None and not content:
            content = " "

        if isinstance(timestamp, datetime):
            timestamp = int(timestamp.timestamp())
        elif timestamp is None:
            timestamp = int(datetime.now(timezone.utc).timestamp())

        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("v2", "users", openid, "messages"),
            json=exclude_none(
                {
                    "msg_type": msg_type,
                    "content": content,
                    "markdown": (
                        markdown.dict(exclude_none=True)
                        if markdown is not None
                        else None
                    ),
                    "keyboard": (
                        keyboard.dict(exclude_none=True)
                        if keyboard is not None
                        else None
                    ),
                    "media": (
                        media.dict(exclude_none=True) if media is not None else None
                    ),
                    "ark": ark.dict(exclude_none=True) if ark is not None else None,
                    "embed": (
                        embed.dict(exclude_none=True) if embed is not None else None
                    ),
                    "image": image,
                    "message_reference": message_reference,
                    "event_id": event_id,
                    "msg_id": msg_id,
                    "msg_seq": msg_seq,
                    "timestamp": timestamp,
                }
            ),
        )
        return type_validate_python(PostC2CMessagesReturn, await self._request(request))

    @API
    async def post_c2c_files(
        self,
        *,
        openid: str,
        file_type: Literal[1, 2, 3, 4],
        url: Optional[str] = None,
        srv_send_msg: bool = True,
        file_data: Optional[Union[str, bytes]] = None,
    ) -> PostC2CFilesReturn:
        if isinstance(file_data, bytes):
            file_data = b64encode(file_data).decode()
        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("v2", "users", openid, "files"),
            json=exclude_none(
                {
                    "file_type": file_type,
                    "url": url,
                    "srv_send_msg": srv_send_msg,
                    "file_data": file_data,
                }
            ),
        )
        return type_validate_python(PostC2CFilesReturn, await self._request(request))

    @API
    async def delete_c2c_message(self, *, openid: str, message_id: str) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath(
                "v2", "users", openid, "messages", message_id
            ),
        )
        return await self._request(request)

    # Group API
    @API
    async def post_group_messages(
        self,
        *,
        group_openid: str,
        msg_type: Literal[0, 1, 2, 3, 4, 7],
        content: Optional[str] = None,
        markdown: Optional[MessageMarkdown] = None,
        keyboard: Optional[MessageKeyboard] = None,
        media: Optional[Media] = None,
        ark: Optional[MessageArk] = None,
        embed: Optional[MessageEmbed] = None,
        image: None = None,
        message_reference: None = None,
        event_id: Optional[str] = None,
        msg_id: Optional[str] = None,
        msg_seq: Optional[int] = None,
        timestamp: Optional[Union[int, datetime]] = None,
    ) -> PostGroupMessagesReturn:
        # tmp fix. content must not be none if sending media
        if media is not None and not content:
            content = " "

        if isinstance(timestamp, datetime):
            timestamp = int(timestamp.timestamp())
        elif timestamp is None:
            timestamp = int(datetime.now(timezone.utc).timestamp())

        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath(
                "v2", "groups", group_openid, "messages"
            ),
            json=exclude_none(
                {
                    "msg_type": msg_type,
                    "content": content,
                    "markdown": (
                        markdown.dict(exclude_none=True)
                        if markdown is not None
                        else None
                    ),
                    "keyboard": (
                        keyboard.dict(exclude_none=True)
                        if keyboard is not None
                        else None
                    ),
                    "media": (
                        media.dict(exclude_none=True) if media is not None else None
                    ),
                    "ark": ark.dict(exclude_none=True) if ark is not None else None,
                    "embed": (
                        embed.dict(exclude_none=True) if embed is not None else None
                    ),
                    "image": image,
                    "message_reference": message_reference,
                    "event_id": event_id,
                    "msg_id": msg_id,
                    "msg_seq": msg_seq,
                    "timestamp": timestamp,
                }
            ),
        )
        return type_validate_python(
            PostGroupMessagesReturn, await self._request(request)
        )

    @API
    async def post_group_files(
        self,
        *,
        group_openid: str,
        file_type: Literal[1, 2, 3, 4],
        url: Optional[str] = None,
        srv_send_msg: bool = True,
        file_data: Optional[Union[str, bytes]] = None,
    ) -> PostGroupFilesReturn:
        if isinstance(file_data, bytes):
            file_data = b64encode(file_data).decode()
        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("v2", "groups", group_openid, "files"),
            json=exclude_none(
                {
                    "file_type": file_type,
                    "url": url,
                    "srv_send_msg": srv_send_msg,
                    "file_data": file_data,
                }
            ),
        )
        return type_validate_python(PostGroupFilesReturn, await self._request(request))

    @API
    async def delete_group_message(self, *, group_openid: str, message_id: str) -> None:
        request = Request(
            "DELETE",
            self.adapter.get_api_base().joinpath(
                "v2", "groups", group_openid, "messages", message_id
            ),
        )
        return await self._request(request)

    @API
    async def post_group_members(
        self,
        *,
        group_id: str,
        limit: Optional[int] = None,
        start_index: Optional[int] = None,
    ) -> PostGroupMembersReturn:
        request = Request(
            "POST",
            self.adapter.get_api_base().joinpath("v2", "groups", group_id, "members"),
            json=exclude_none({"limit": limit, "start_index": start_index}),
        )
        return type_validate_python(
            PostGroupMembersReturn, await self._request(request)
        )



import json
from typing import Optional

from nonebot.drivers import Response
from nonebot.exception import AdapterException
from nonebot.exception import ActionFailed as BaseActionFailed
from nonebot.exception import NetworkError as BaseNetworkError
from nonebot.exception import NoLogException as BaseNoLogException
from nonebot.exception import ApiNotAvailable as BaseApiNotAvailable

from .store import audit_result


class QQAdapterException(AdapterException):
    def __init__(self):
        super().__init__("qq")


class NoLogException(BaseNoLogException, QQAdapterException):
    pass


class ActionFailed(BaseActionFailed, QQAdapterException):
    def __init__(self, response: Response):
        self.response = response

        self.body: Optional[dict] = None
        if response.content:
            try:
                self.body = json.loads(response.content)
            except Exception:
                pass

    @property
    def status_code(self) -> int:
        return self.response.status_code

    @property
    def code(self) -> Optional[int]:
        return None if self.body is None else self.body.get("code", None)

    @property
    def message(self) -> Optional[str]:
        return None if self.body is None else self.body.get("message", None)

    @property
    def data(self) -> Optional[dict]:
        return None if self.body is None else self.body.get("data", None)

    @property
    def trace_id(self) -> Optional[str]:
        return self.response.headers.get("X-Tps-trace-ID", None)

    def __repr__(self) -> str:
        args = ("code", "message", "data", "trace_id")
        return (
            f"<ActionFailed: {self.status_code}, "
            + ", ".join(f"{k}={v}" for k in args if (v := getattr(self, k)) is not None)
            + ">"
        )

    def __str__(self):
        return self.__repr__()


class UnauthorizedException(ActionFailed):
    pass


class RateLimitException(ActionFailed):
    pass


class NetworkError(BaseNetworkError, QQAdapterException):
    def __init__(self, msg: Optional[str] = None):
        super().__init__()
        self.msg: Optional[str] = msg
        """错误原因"""

    def __repr__(self):
        return f"<NetWorkError message={self.msg}>"

    def __str__(self):
        return self.__repr__()


class ApiNotAvailable(BaseApiNotAvailable, QQAdapterException):
    pass


class AuditException(QQAdapterException):
    """消息审核"""

    def __init__(self, audit_id: str):
        super().__init__()
        self.audit_id = audit_id

    async def get_audit_result(self, timeout: Optional[float] = None):
        """获取审核结果"""
        return await audit_result.fetch(self.audit_id, timeout)



import re
from io import BytesIO
from pathlib import Path
from dataclasses import dataclass
from collections.abc import Iterable
from typing_extensions import Self, override
from typing import TYPE_CHECKING, Union, Optional, TypedDict, overload

from nonebot.compat import type_validate_python

from nonebot.adapters import Message as BaseMessage
from nonebot.adapters import MessageSegment as BaseMessageSegment

from .models import QQMessage
from .utils import escape, unescape
from .models import Message as GuildMessage
from .models import Attachment as QQAttachment
from .models import (
    MessageArk,
    MessageEmbed,
    MessageKeyboard,
    MessageMarkdown,
    MessageReference,
)


class MessageSegment(BaseMessageSegment["Message"]):
    @classmethod
    @override
    def get_message_class(cls) -> type["Message"]:
        return Message

    @staticmethod
    def text(content: str) -> "Text":
        return Text("text", {"text": content})

    @staticmethod
    def emoji(id: str) -> "Emoji":
        return Emoji("emoji", data={"id": id})

    @staticmethod
    def mention_user(user_id: str) -> "MentionUser":
        return MentionUser("mention_user", {"user_id": str(user_id)})

    @staticmethod
    def mention_channel(channel_id: str) -> "MentionChannel":
        return MentionChannel("mention_channel", {"channel_id": str(channel_id)})

    @staticmethod
    def mention_everyone() -> "MentionEveryone":
        return MentionEveryone("mention_everyone", {})

    @staticmethod
    def image(url: str) -> "Attachment":
        return Attachment("image", data={"url": url})

    @staticmethod
    def file_image(data: Union[bytes, BytesIO, Path]) -> "LocalAttachment":
        if isinstance(data, BytesIO):
            data = data.getvalue()
        elif isinstance(data, Path):
            data = data.read_bytes()
        return LocalAttachment("file_image", data={"content": data})

    @staticmethod
    def audio(url: str) -> "Attachment":
        return Attachment("audio", data={"url": url})

    @staticmethod
    def file_audio(data: Union[bytes, BytesIO, Path]) -> "LocalAttachment":
        if isinstance(data, BytesIO):
            data = data.getvalue()
        elif isinstance(data, Path):
            data = data.read_bytes()
        return LocalAttachment("file_audio", data={"content": data})

    @staticmethod
    def video(url: str) -> "Attachment":
        return Attachment("video", data={"url": url})

    @staticmethod
    def file_video(data: Union[bytes, BytesIO, Path]) -> "LocalAttachment":
        if isinstance(data, BytesIO):
            data = data.getvalue()
        elif isinstance(data, Path):
            data = data.read_bytes()
        return LocalAttachment("file_video", data={"content": data})

    @staticmethod
    def file(url: str) -> "Attachment":
        return Attachment("file", data={"url": url})

    @staticmethod
    def file_file(data: Union[bytes, BytesIO, Path]) -> "LocalAttachment":
        if isinstance(data, BytesIO):
            data = data.getvalue()
        elif isinstance(data, Path):
            data = data.read_bytes()
        return LocalAttachment("file_file", data={"content": data})

    @staticmethod
    def ark(ark: MessageArk) -> "Ark":
        return Ark("ark", data={"ark": ark})

    @staticmethod
    def embed(embed: MessageEmbed) -> "Embed":
        return Embed("embed", data={"embed": embed})

    @staticmethod
    def markdown(markdown: Union[str, MessageMarkdown]) -> "Markdown":
        return Markdown(
            "markdown",
            data={
                "markdown": (
                    MessageMarkdown(content=markdown)
                    if isinstance(markdown, str)
                    else markdown
                )
            },
        )

    @staticmethod
    def keyboard(keyboard: MessageKeyboard) -> "Keyboard":
        return Keyboard("keyboard", data={"keyboard": keyboard})

    @overload
    @staticmethod
    def reference(reference: MessageReference) -> "Reference": ...

    @overload
    @staticmethod
    def reference(
        reference: str, ignore_error: Optional[bool] = None
    ) -> "Reference": ...

    @staticmethod
    def reference(
        reference: Union[str, MessageReference], ignore_error: Optional[bool] = None
    ) -> "Reference":
        if isinstance(reference, MessageReference):
            return Reference("reference", data={"reference": reference})

        return Reference(
            "reference",
            data={
                "reference": MessageReference(
                    message_id=reference, ignore_get_message_error=ignore_error
                )
            },
        )

    @override
    def __add__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return Message(self) + (
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @override
    def __radd__(
        self, other: Union[str, "MessageSegment", Iterable["MessageSegment"]]
    ) -> "Message":
        return (
            MessageSegment.text(other) if isinstance(other, str) else Message(other)
        ) + self

    @override
    def is_text(self) -> bool:
        return self.type == "text"

    @classmethod
    @override
    def _validate(cls, value) -> Self:
        if isinstance(value, cls):
            return value
        if isinstance(value, MessageSegment):
            raise ValueError(f"Type {type(value)} can not be converted to {cls}")
        if not isinstance(value, dict):
            raise ValueError(f"Expected dict for MessageSegment, got {type(value)}")
        if "type" not in value:
            raise ValueError(
                f"Expected dict with 'type' for MessageSegment, got {value}"
            )
        _type = value["type"]
        if _type not in SEGMENT_TYPE_MAP:
            raise ValueError(f"Invalid MessageSegment type: {_type}")
        segment_type = SEGMENT_TYPE_MAP[_type]

        # casting value to subclass of MessageSegment
        if cls is MessageSegment:
            return type_validate_python(segment_type, value)
        # init segment instance directly if type matched
        if cls is segment_type:
            return segment_type(type=_type, data=value.get("data", {}))
        raise ValueError(f"Segment type {_type!r} can not be converted to {cls}")


class _TextData(TypedDict):
    text: str


@dataclass
class Text(MessageSegment):
    if TYPE_CHECKING:
        data: _TextData

    @override
    def __str__(self) -> str:
        return escape(self.data["text"])


class _EmojiData(TypedDict):
    id: str


@dataclass
class Emoji(MessageSegment):
    if TYPE_CHECKING:
        data: _EmojiData

    @override
    def __str__(self) -> str:
        return f"<emoji:{self.data['id']}>"


class _MentionUserData(TypedDict):
    user_id: str


@dataclass
class MentionUser(MessageSegment):
    if TYPE_CHECKING:
        data: _MentionUserData

    @override
    def __str__(self) -> str:
        return f"<@{self.data['user_id']}>"


class _MentionChannelData(TypedDict):
    channel_id: str


@dataclass
class MentionChannel(MessageSegment):
    if TYPE_CHECKING:
        data: _MentionChannelData

    @override
    def __str__(self) -> str:
        return f"<#{self.data['channel_id']}>"


class _MentionEveryoneData(TypedDict):
    pass


@dataclass
class MentionEveryone(MessageSegment):
    if TYPE_CHECKING:
        data: _MentionEveryoneData

    @override
    def __str__(self) -> str:
        return "@everyone"


class _AttachmentData(TypedDict):
    url: str


@dataclass
class Attachment(MessageSegment):
    if TYPE_CHECKING:
        data: _AttachmentData

    @override
    def __str__(self) -> str:
        return f"<attachment[{self.type}]:{self.data['url']}>"


class _LocalAttachmentData(TypedDict):
    content: bytes


@dataclass
class LocalAttachment(MessageSegment):
    if TYPE_CHECKING:
        data: _LocalAttachmentData

    @override
    def __str__(self) -> str:
        return f"<local_attachment[{self.type}]>"


class _EmbedData(TypedDict):
    embed: MessageEmbed


@dataclass
class Embed(MessageSegment):
    if TYPE_CHECKING:
        data: _EmbedData

    @override
    def __str__(self) -> str:
        return f"<embed:{self.data['embed']!r}>"

    @classmethod
    @override
    def _validate(cls, value) -> Self:
        instance = super()._validate(value)
        if "embed" not in instance.data:
            raise ValueError(
                f"Expected dict with 'embed' in 'data' for Embed, got {value}"
            )
        if not isinstance(embed := instance.data["embed"], MessageEmbed):
            instance.data["embed"] = type_validate_python(MessageEmbed, embed)
        return instance


class _ArkData(TypedDict):
    ark: MessageArk


@dataclass
class Ark(MessageSegment):
    if TYPE_CHECKING:
        data: _ArkData

    @override
    def __str__(self) -> str:
        return f"<ark:{self.data['ark']!r}>"

    @classmethod
    @override
    def _validate(cls, value) -> Self:
        instance = super()._validate(value)
        if "ark" not in instance.data:
            raise ValueError(f"Expected dict with 'ark' in 'data' for Ark, got {value}")
        if not isinstance(ark := instance.data["ark"], MessageArk):
            instance.data["ark"] = type_validate_python(MessageArk, ark)
        return instance


class _ReferenceData(TypedDict):
    reference: MessageReference


@dataclass
class Reference(MessageSegment):
    if TYPE_CHECKING:
        data: _ReferenceData

    @override
    def __str__(self) -> str:
        return f"<reference:{self.data['reference'].message_id}>"

    @classmethod
    @override
    def _validate(cls, value) -> Self:
        instance = super()._validate(value)
        if "reference" not in instance.data:
            raise ValueError(
                f"Expected dict with 'reference' in 'data' for Reference, got {value}"
            )
        if not isinstance(reference := instance.data["reference"], MessageReference):
            instance.data["reference"] = type_validate_python(
                MessageReference, reference
            )
        return instance


class _MarkdownData(TypedDict):
    markdown: MessageMarkdown


@dataclass
class Markdown(MessageSegment):
    if TYPE_CHECKING:
        data: _MarkdownData

    @override
    def __str__(self) -> str:
        return f"<markdown:{self.data['markdown']!r}>"

    @classmethod
    @override
    def _validate(cls, value) -> Self:
        instance = super()._validate(value)
        if "markdown" not in instance.data:
            raise ValueError(
                f"Expected dict with 'markdown' in 'data' for Markdown, got {value}"
            )
        if not isinstance(markdown := instance.data["markdown"], MessageMarkdown):
            instance.data["markdown"] = type_validate_python(MessageMarkdown, markdown)
        return instance


class _KeyboardData(TypedDict):
    keyboard: MessageKeyboard


@dataclass
class Keyboard(MessageSegment):
    if TYPE_CHECKING:
        data: _KeyboardData

    @override
    def __str__(self) -> str:
        return f"<keyboard:{self.data['keyboard']!r}>"

    @classmethod
    @override
    def _validate(cls, value) -> Self:
        instance = super()._validate(value)
        if "keyboard" not in instance.data:
            raise ValueError(
                f"Expected dict with 'keyboard' in 'data' for Keyboard, got {value}"
            )
        if not isinstance(keyboard := instance.data["keyboard"], MessageKeyboard):
            instance.data["keyboard"] = type_validate_python(MessageKeyboard, keyboard)
        return instance


SEGMENT_TYPE_MAP: dict[str, type[MessageSegment]] = {
    "text": Text,
    "emoji": Emoji,
    "mention_user": MentionUser,
    "mention_channel": MentionChannel,
    "mention_everyone": MentionEveryone,
    "image": Attachment,
    "file_image": LocalAttachment,
    "audio": Attachment,
    "file_audio": LocalAttachment,
    "video": Attachment,
    "file_video": LocalAttachment,
    "file": Attachment,
    "file_file": LocalAttachment,
    "ark": Ark,
    "embed": Embed,
    "markdown": Markdown,
    "keyboard": Keyboard,
    "reference": Reference,
}


class Message(BaseMessage[MessageSegment]):
    @classmethod
    @override
    def get_segment_class(cls) -> type[MessageSegment]:
        return MessageSegment

    @override
    def __add__(
        self, other: Union[str, MessageSegment, Iterable[MessageSegment]]
    ) -> Self:
        return super().__add__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @override
    def __radd__(
        self, other: Union[str, MessageSegment, Iterable[MessageSegment]]
    ) -> Self:
        return super().__radd__(
            MessageSegment.text(other) if isinstance(other, str) else other
        )

    @staticmethod
    @override
    def _construct(msg: str) -> Iterable[MessageSegment]:
        text_begin = 0
        for embed in re.finditer(
            r"\<(?P<type>(?:@|#|emoji:))!?(?P<id>\w+?)\>",
            msg,
        ):
            content = msg[text_begin : embed.pos + embed.start()]
            if content:
                yield Text("text", {"text": unescape(content)})
            text_begin = embed.pos + embed.end()
            if embed.group("type") == "@":
                yield MentionUser("mention_user", {"user_id": embed.group("id")})
            elif embed.group("type") == "#":
                yield MentionChannel(
                    "mention_channel", {"channel_id": embed.group("id")}
                )
            else:
                yield Emoji("emoji", {"id": embed.group("id")})
        content = msg[text_begin:]
        if content:
            yield Text("text", {"text": unescape(msg[text_begin:])})

    @classmethod
    def from_guild_message(cls, message: GuildMessage) -> Self:
        msg = cls()
        if message.mention_everyone:
            msg.append(MessageSegment.mention_everyone())
        if message.content:
            msg.extend(Message(message.content))
        if message.attachments:
            msg.extend(
                MessageSegment.image(seg.url) for seg in message.attachments if seg.url
            )
        if message.embeds:
            msg.extend(Embed("embed", data={"embed": seg}) for seg in message.embeds)
        if message.ark:
            msg.append(Ark("ark", data={"ark": message.ark}))
        return msg

    @classmethod
    def from_qq_message(cls, message: QQMessage) -> Self:
        msg = cls()
        if message.content:
            msg.extend(Message(message.content))
        if message.attachments:

            def content_type(seg: QQAttachment):
                ct = seg.content_type.split("/", maxsplit=1)[0]
                if ct in {"image", "audio", "file", "video"}:
                    return ct
                return "file"

            msg.extend(
                Attachment(
                    content_type(seg),
                    data={"url": seg.url},
                )
                for seg in message.attachments
                if seg.url
            )
        return msg

    def extract_content(self) -> str:
        return "".join(
            str(seg)
            for seg in self
            if seg.type
            in ("text", "emoji", "mention_user", "mention_everyone", "mention_channel")
        )

    @override
    def extract_plain_text(self) -> str:
        return "".join(seg.data["text"] for seg in self if seg.is_text())



import json
from enum import IntEnum
from datetime import datetime
from typing import Union, Generic, TypeVar, Optional

from pydantic import BaseModel
from nonebot.compat import PYDANTIC_V2, model_fields, type_validate_python

from nonebot.adapters.qq.compat import field_validator, model_validator

from .common import MessageArk, MessageEmbed, MessageReference, MessageAttachment

if PYDANTIC_V2:
    GenericModel = BaseModel
else:
    from pydantic.generics import GenericModel

T = TypeVar("T")


# Guild
class Guild(BaseModel):
    id: str
    name: str
    icon: str
    owner_id: str
    owner: bool
    member_count: int
    max_members: int
    description: str
    joined_at: datetime


# User
class User(BaseModel):
    id: str
    username: Optional[str] = None
    avatar: Optional[str] = None
    bot: Optional[bool] = None
    union_openid: Optional[str] = None
    union_user_account: Optional[str] = None


# Channel
class ChannelType(IntEnum):
    TEXT = 0  # 文字子频道
    # x = 1    # 保留，不可用
    VOICE = 2  # 语音子频道
    # x = 3    # 保留，不可用
    GROUP = 4  # 子频道分组
    LIVE = 10005  # 直播子频道
    APP = 10006  # 应用子频道
    DISCUSSION = 10007  # 论坛子频道


class ChannelSubType(IntEnum):
    TALK = 0  # 闲聊
    POST = 1  # 公告
    CHEAT = 2  # 攻略
    BLACK = 3  # 开黑


class PrivateType(IntEnum):
    PUBLIC = 0  # 公开频道
    ADMIN = 1  # 管理员和群主可见
    SPECIFIED_USER = 2  # 群主管理员+指定成员，可使用 修改子频道权限接口 指定成员


class SpeakPermission(IntEnum):
    INVALID = 0  # 无效类型
    EVERYONE = 1  # 所有人
    ADMIN = 2  # 群主管理员+指定成员，可使用 修改子频道权限接口 指定成员


class Channel(BaseModel):
    id: str
    guild_id: str
    name: str
    type: Union[ChannelType, int]
    sub_type: Union[ChannelSubType, int]
    position: int
    parent_id: Optional[str] = None
    owner_id: Optional[str] = None
    private_type: Optional[Union[PrivateType, int]] = None
    speak_permission: Optional[Union[SpeakPermission, int]] = None
    application_id: Optional[str] = None
    permissions: Optional[int] = None


# Member
class Member(BaseModel):
    user: Optional[User] = None
    nick: Optional[str] = None
    roles: Optional[list[str]] = None
    joined_at: datetime


class GetRoleMembersReturn(BaseModel):
    data: list[Member]
    next: str


# Role
class Role(BaseModel):
    id: str
    name: str
    color: int
    hoist: bool
    number: int
    member_limit: int


class GetGuildRolesReturn(BaseModel):
    guild_id: str
    roles: list[Role]
    role_num_limit: int


class PostGuildRoleReturn(BaseModel):
    role_id: str
    role: Role


class PatchGuildRoleReturn(BaseModel):
    guild_id: str
    role_id: str
    role: Role


# Channel Permission
class ChannelPermissions(BaseModel):
    channel_id: str
    user_id: Optional[str] = None
    role_id: Optional[str] = None
    permissions: int


# Message
class Message(BaseModel):
    id: str
    channel_id: str
    guild_id: str
    content: Optional[str] = None
    timestamp: Optional[datetime] = None
    edited_timestamp: Optional[datetime] = None
    mention_everyone: Optional[bool] = None
    author: User
    attachments: Optional[list[MessageAttachment]] = None
    embeds: Optional[list[MessageEmbed]] = None
    mentions: Optional[list[User]] = None
    member: Optional[Member] = None
    ark: Optional[MessageArk] = None
    seq: Optional[int] = None
    seq_in_channel: Optional[str] = None
    message_reference: Optional[MessageReference] = None
    src_guild_id: Optional[str] = None


# Message Delete Event
class MessageDelete(BaseModel):
    message: Message
    op_user: User


# Message Setting
class MessageSetting(BaseModel):
    disable_create_dm: bool
    disable_push_msg: bool
    channel_ids: list[str]
    channel_push_max_num: int


# DMS
class DMS(BaseModel):
    guild_id: Optional[str] = None
    channel_id: Optional[str] = None
    create_time: Optional[datetime] = None


# Announce
class RecommendChannel(BaseModel):
    channel_id: Optional[str] = None
    introduce: Optional[str] = None


class Announces(BaseModel):
    guild_id: Optional[str] = None
    channel_id: Optional[str] = None
    message_id: Optional[str] = None
    announces_type: Optional[int] = None
    recommend_channels: Optional[list[RecommendChannel]] = None


# Pins
class PinsMessage(BaseModel):
    guild_id: str
    channel_id: str
    message_ids: list[str]


# Schedule
class RemindType(IntEnum):
    NO_REMIND = 0
    REMIND_START = 1
    REMIND_5_MIN = 2
    REMIND_15_MIN = 3
    REMIND_30_MIN = 4
    REMIND_60_MIN = 5


class Schedule(BaseModel):
    id: str
    name: str
    description: Optional[str] = None
    start_timestamp: datetime
    end_timestamp: datetime
    creator: Optional[Member] = None
    jump_channel_id: Optional[str] = None
    remind_type: Optional[Union[RemindType, int]] = None


# Emoji Reaction
class EmojiType(IntEnum):
    SYSTEM = 1
    CUSTOM = 2


class Emoji(BaseModel):
    id: str
    type: int


class ReactionTargetType(IntEnum):
    MESSAGE = 0
    FEED = 1
    COMMENT = 2
    REPLY = 3


class ReactionTarget(BaseModel):
    id: str
    type: Union[ReactionTargetType, str]


class MessageReaction(BaseModel):
    user_id: str
    guild_id: str
    channel_id: str
    target: ReactionTarget
    emoji: Emoji


class GetReactionUsersReturn(BaseModel):
    users: list[User]
    cookie: str
    is_end: bool


# Audio
class AudioStatus(IntEnum):
    START = 0
    PAUSE = 1
    RESUME = 2
    STOP = 3


class AudioControl(BaseModel):
    audio_url: Optional[str] = None
    text: Optional[str] = None
    status: Union[AudioStatus, int]


class AudioAction(BaseModel):
    guild_id: str
    channel_id: str
    audio_url: Optional[str] = None
    text: Optional[str] = None


# Forum
class ElemType(IntEnum):
    UNSUPPORTED = 0
    TEXT = 1
    IMAGE = 2
    VIDEO = 3
    URL = 4


class TextProps(BaseModel):
    font_bold: Optional[bool] = None
    italic: Optional[bool] = None
    underline: Optional[bool] = None


class TextElem(BaseModel):
    text: str
    props: Optional[TextProps] = None


class PlatImage(BaseModel):
    url: Optional[str] = None
    width: Optional[int] = None
    height: Optional[int] = None
    image_id: Optional[str] = None


class ImageElem(BaseModel):
    plat_image: Optional[PlatImage] = None
    third_url: Optional[str] = None
    width_percent: Optional[float] = None


class VideoElem(BaseModel):
    third_url: str


class URLElem(BaseModel):
    url: str
    desc: Optional[str] = None


class Elem(BaseModel):
    type: ElemType
    text: Optional[TextElem] = None
    image: Optional[ImageElem] = None
    video: Optional[VideoElem] = None
    url: Optional[URLElem] = None

    @model_validator(mode="before")
    @classmethod
    def infer_type(cls, values: dict):
        if values.get("type") is not None:
            return values

        if values.get("text") is not None:
            values["type"] = ElemType.TEXT
        elif values.get("image") is not None:
            values["type"] = ElemType.IMAGE
        elif values.get("video") is not None:
            values["type"] = ElemType.VIDEO
        elif values.get("url") is not None:
            values["type"] = ElemType.URL
        else:
            # Unsupported element type
            values["type"] = ElemType.UNSUPPORTED

        return values


class Alignment(IntEnum):
    LEFT = 0
    MIDDLE = 1
    RIGHT = 2


class ParagraphProps(BaseModel):
    alignment: Optional[Alignment] = None


class Paragraph(BaseModel):
    elems: list[Elem]
    props: Optional[ParagraphProps] = None


class RichText(BaseModel):
    paragraphs: list[Paragraph]


class ThreadObjectInfo(BaseModel):
    thread_id: str
    content: RichText
    date_time: datetime

    @field_validator("content", mode="before")
    @classmethod
    def parse_content(cls, v):
        if isinstance(v, str):
            return type_validate_python(RichText, json.loads(v))
        return v


_T_Title = TypeVar("_T_Title", str, RichText)


class ThreadInfo(ThreadObjectInfo, GenericModel, Generic[_T_Title]):
    # 事件推送拿到的title实际上是RichText的JSON字符串，而API调用返回的title是普通文本
    title: _T_Title

    @field_validator("title", mode="before")
    @classmethod
    def parse_title(cls, v):
        if (
            isinstance(v, str)
            and next(f for f in model_fields(cls) if f.name == "title").annotation
            is RichText
        ):
            return type_validate_python(RichText, json.loads(v))
        return v


class ForumSourceInfo(BaseModel):
    guild_id: str
    channel_id: str
    author_id: str


class Thread(ForumSourceInfo, GenericModel, Generic[_T_Title]):
    thread_info: ThreadInfo[_T_Title]


class PostInfo(ThreadObjectInfo):
    post_id: str


class Post(ForumSourceInfo):
    post_info: PostInfo


class ReplyInfo(ThreadObjectInfo):
    post_id: str
    reply_id: str


class Reply(ForumSourceInfo):
    reply_info: ReplyInfo


class ForumAuditType(IntEnum):
    THREAD = 1
    POST = 2
    REPLY = 3


class ForumAuditResult(ForumSourceInfo):
    thread_id: str
    post_id: str
    reply_id: str
    type: ForumAuditType
    result: Optional[int] = None
    err_msg: Optional[str] = None


class GetThreadsListReturn(BaseModel):
    threads: list[Thread[str]]
    is_finish: bool


class GetThreadReturn(BaseModel):
    thread: Thread[str]


class PutThreadReturn(BaseModel):
    task_id: str
    create_time: datetime


# API Permission
class APIPermission(BaseModel):
    path: str
    method: str
    desc: Optional[str] = None
    auth_status: bool


class APIPermissionDemandIdentify(BaseModel):
    path: Optional[str] = None
    method: Optional[str] = None


class APIPermissionDemand(BaseModel):
    guild_id: str
    channel_id: str
    api_identify: APIPermissionDemandIdentify
    title: str
    desc: str


class GetGuildAPIPermissionReturn(BaseModel):
    apis: list[APIPermission]


# WebSocket
class UrlGetReturn(BaseModel):
    url: str


class SessionStartLimit(BaseModel):
    total: int
    remaining: int
    reset_after: int
    max_concurrency: int


class ShardUrlGetReturn(BaseModel):
    url: str
    shards: int
    session_start_limit: SessionStartLimit


__all__ = [
    "Guild",
    "User",
    "ChannelType",
    "ChannelSubType",
    "PrivateType",
    "SpeakPermission",
    "Channel",
    "Member",
    "GetRoleMembersReturn",
    "Role",
    "GetGuildRolesReturn",
    "PostGuildRoleReturn",
    "PatchGuildRoleReturn",
    "ChannelPermissions",
    "Message",
    "MessageDelete",
    "MessageSetting",
    "DMS",
    "RecommendChannel",
    "Announces",
    "PinsMessage",
    "RemindType",
    "Schedule",
    "EmojiType",
    "Emoji",
    "ReactionTargetType",
    "ReactionTarget",
    "MessageReaction",
    "GetReactionUsersReturn",
    "AudioStatus",
    "AudioControl",
    "AudioAction",
    "ElemType",
    "TextProps",
    "TextElem",
    "ImageElem",
    "VideoElem",
    "URLElem",
    "Elem",
    "Alignment",
    "ParagraphProps",
    "Paragraph",
    "RichText",
    "ThreadObjectInfo",
    "ThreadInfo",
    "ForumSourceInfo",
    "Thread",
    "PostInfo",
    "Post",
    "ReplyInfo",
    "Reply",
    "ForumAuditType",
    "ForumAuditResult",
    "GetThreadsListReturn",
    "GetThreadReturn",
    "PutThreadReturn",
    "APIPermission",
    "APIPermissionDemandIdentify",
    "APIPermissionDemand",
    "GetGuildAPIPermissionReturn",
    "UrlGetReturn",
    "SessionStartLimit",
    "ShardUrlGetReturn",
]



from datetime import datetime
from urllib.parse import urlparse
from typing import Literal, Optional

from pydantic import BaseModel

from nonebot.adapters.qq.compat import field_validator


# Message Attachment
class MessageAttachment(BaseModel):
    url: str

    @field_validator("url", mode="after")
    @classmethod
    def check_url(cls, v: str):
        if v and not urlparse(v).hostname:
            return f"https://{v}"
        return v


# Message Embed
class MessageEmbedThumbnail(BaseModel):
    url: str


class MessageEmbedField(BaseModel):
    name: Optional[str] = None


class MessageEmbed(BaseModel):
    title: Optional[str] = None
    description: Optional[str] = None
    prompt: str
    thumbnail: Optional[MessageEmbedThumbnail] = None
    fields: Optional[list[MessageEmbedField]] = None


# Message Ark
class MessageArkObjKv(BaseModel):
    key: Optional[str] = None
    value: Optional[str] = None


class MessageArkObj(BaseModel):
    obj_kv: Optional[list[MessageArkObjKv]] = None


class MessageArkKv(BaseModel):
    key: Optional[str] = None
    value: Optional[str] = None
    obj: Optional[list[MessageArkObj]] = None


class MessageArk(BaseModel):
    template_id: Optional[int] = None
    kv: Optional[list[MessageArkKv]] = None


# Message Reference
class MessageReference(BaseModel):
    message_id: str
    ignore_get_message_error: Optional[bool] = None


# Message Markdown
class MessageMarkdownParams(BaseModel):
    key: Optional[str] = None
    values: Optional[list[str]] = None


class MessageMarkdown(BaseModel):
    template_id: Optional[int] = None
    custom_template_id: Optional[str] = None
    params: Optional[list[MessageMarkdownParams]] = None
    content: Optional[str] = None


# Message Keyboard
class Permission(BaseModel):
    type: Optional[int] = None
    specify_role_ids: Optional[list[str]] = None
    specify_user_ids: Optional[list[str]] = None


class Action(BaseModel):
    type: Optional[int] = None
    permission: Optional[Permission] = None
    data: Optional[str] = None
    reply: Optional[bool] = None
    enter: Optional[bool] = None
    anchor: Optional[int] = None
    unsupport_tips: Optional[str] = None
    click_limit: Optional[int] = None  # deprecated
    at_bot_show_channel_list: Optional[bool] = None  # deprecated


class RenderData(BaseModel):
    label: Optional[str] = None
    visited_label: Optional[str] = None
    style: Optional[int] = None


class Button(BaseModel):
    id: Optional[str] = None
    render_data: Optional[RenderData] = None
    action: Optional[Action] = None


class InlineKeyboardRow(BaseModel):
    buttons: Optional[list[Button]] = None


class InlineKeyboard(BaseModel):
    rows: Optional[list[InlineKeyboardRow]] = None


class MessageKeyboard(BaseModel):
    id: Optional[str] = None
    content: Optional[InlineKeyboard] = None


# Message Audit Event
class MessageAudited(BaseModel):
    audit_id: str
    message_id: Optional[str] = None
    user_openid: Optional[str] = None
    group_openid: Optional[str] = None
    guild_id: Optional[str] = None
    channel_id: Optional[str] = None
    audit_time: datetime
    create_time: Optional[datetime] = None
    seq_in_channel: Optional[str] = None


# Interaction Event
class ButtonInteractionContent(BaseModel):
    user_id: Optional[str] = None
    message_id: Optional[str] = None
    feature_id: Optional[str] = None
    button_id: Optional[str] = None
    button_data: Optional[str] = None
    checked: Optional[int] = None
    feedback_opt: Optional[str] = None


class ButtonInteractionData(BaseModel):
    resolved: ButtonInteractionContent


class ButtonInteraction(BaseModel):
    id: str
    type: Literal[11, 12, 13]
    version: int
    timestamp: str
    scene: str
    chat_type: int
    guild_id: Optional[str] = None
    channel_id: Optional[str] = None
    user_openid: Optional[str] = None
    group_openid: Optional[str] = None
    group_member_openid: Optional[str] = None
    application_id: str
    data: ButtonInteractionData


__all__ = [
    "MessageAttachment",
    "MessageEmbedThumbnail",
    "MessageEmbedField",
    "MessageEmbed",
    "MessageArkObjKv",
    "MessageArkObj",
    "MessageArkKv",
    "MessageArk",
    "MessageReference",
    "MessageMarkdownParams",
    "MessageMarkdown",
    "Permission",
    "Action",
    "RenderData",
    "Button",
    "InlineKeyboardRow",
    "InlineKeyboard",
    "MessageKeyboard",
    "MessageAudited",
    "ButtonInteractionContent",
    "ButtonInteractionData",
    "ButtonInteraction",
]



from enum import Enum
from datetime import datetime
from typing_extensions import override
from typing import TypeVar, Optional, cast

from nonebot.utils import escape_tag

from nonebot.adapters import Event as BaseEvent

from .message import Message
from .models import Message as GuildMessage
from .models import Post, User, Guild, Reply, Member, Thread, Channel
from .models import (
    RichText,
    QQMessage,
    AudioAction,
    FriendAuthor,
    MessageDelete,
    MessageAudited,
    ForumSourceInfo,
    MessageReaction,
    ForumAuditResult,
    ButtonInteraction,
    GroupMemberAuthor,
)

E = TypeVar("E", bound="Event")


class EventType(str, Enum):
    # Init Event
    READY = "READY"
    RESUMED = "RESUMED"

    # GUILDS
    GUILD_CREATE = "GUILD_CREATE"
    GUILD_UPDATE = "GUILD_UPDATE"
    GUILD_DELETE = "GUILD_DELETE"
    CHANNEL_CREATE = "CHANNEL_CREATE"
    CHANNEL_UPDATE = "CHANNEL_UPDATE"
    CHANNEL_DELETE = "CHANNEL_DELETE"

    # GUILD_MEMBERS
    GUILD_MEMBER_ADD = "GUILD_MEMBER_ADD"
    GUILD_MEMBER_UPDATE = "GUILD_MEMBER_UPDATE"
    GUILD_MEMBER_REMOVE = "GUILD_MEMBER_REMOVE"

    # GUILD_MESSAGES
    MESSAGE_CREATE = "MESSAGE_CREATE"
    MESSAGE_DELETE = "MESSAGE_DELETE"

    # GUILD_MESSAGE_REACTIONS
    MESSAGE_REACTION_ADD = "MESSAGE_REACTION_ADD"
    MESSAGE_REACTION_REMOVE = "MESSAGE_REACTION_REMOVE"

    # DIRECT_MESSAGE
    DIRECT_MESSAGE_CREATE = "DIRECT_MESSAGE_CREATE"
    DIRECT_MESSAGE_DELETE = "DIRECT_MESSAGE_DELETE"

    # OPEN_FORUMS_EVENT
    OPEN_FORUM_THREAD_CREATE = "OPEN_FORUM_THREAD_CREATE"
    OPEN_FORUM_THREAD_UPDATE = "OPEN_FORUM_THREAD_UPDATE"
    OPEN_FORUM_THREAD_DELETE = "OPEN_FORUM_THREAD_DELETE"
    OPEN_FORUM_POST_CREATE = "OPEN_FORUM_POST_CREATE"
    OPEN_FORUM_POST_DELETE = "OPEN_FORUM_POST_DELETE"
    OPEN_FORUM_REPLY_CREATE = "OPEN_FORUM_REPLY_CREATE"
    OPEN_FORUM_REPLY_DELETE = "OPEN_FORUM_REPLY_DELETE"

    # AUDIO_OR_LIVE_CHANNEL_MEMBER
    AUDIO_OR_LIVE_CHANNEL_MEMBER_ENTER = "AUDIO_OR_LIVE_CHANNEL_MEMBER_ENTER"
    AUDIO_OR_LIVE_CHANNEL_MEMBER_EXIT = "AUDIO_OR_LIVE_CHANNEL_MEMBER_EXIT"

    # C2C_GROUP_AT_MESSAGES
    C2C_MESSAGE_CREATE = "C2C_MESSAGE_CREATE"
    GROUP_AT_MESSAGE_CREATE = "GROUP_AT_MESSAGE_CREATE"

    # INTERACTION
    INTERACTION_CREATE = "INTERACTION_CREATE"

    # MESSAGE_AUDIT
    MESSAGE_AUDIT_PASS = "MESSAGE_AUDIT_PASS"
    MESSAGE_AUDIT_REJECT = "MESSAGE_AUDIT_REJECT"

    # FORUM_EVENT
    FORUM_THREAD_CREATE = "FORUM_THREAD_CREATE"
    FORUM_THREAD_UPDATE = "FORUM_THREAD_UPDATE"
    FORUM_THREAD_DELETE = "FORUM_THREAD_DELETE"
    FORUM_POST_CREATE = "FORUM_POST_CREATE"
    FORUM_POST_DELETE = "FORUM_POST_DELETE"
    FORUM_REPLY_CREATE = "FORUM_REPLY_CREATE"
    FORUM_REPLY_DELETE = "FORUM_REPLY_DELETE"
    FORUM_PUBLISH_AUDIT_RESULT = "FORUM_PUBLISH_AUDIT_RESULT"

    # AUDIO_ACTION
    AUDIO_START = "AUDIO_START"
    AUDIO_FINISH = "AUDIO_FINISH"
    AUDIO_ON_MIC = "AUDIO_ON_MIC"
    AUDIO_OFF_MIC = "AUDIO_OFF_MIC"

    # AT_MESSAGES
    AT_MESSAGE_CREATE = "AT_MESSAGE_CREATE"
    PUBLIC_MESSAGE_DELETE = "PUBLIC_MESSAGE_DELETE"

    # FRIEND_ROBOT_EVENT
    FRIEND_ADD = "FRIEND_ADD"
    FRIEND_DEL = "FRIEND_DEL"
    C2C_MSG_REJECT = "C2C_MSG_REJECT"
    C2C_MSG_RECEIVE = "C2C_MSG_RECEIVE"

    # GROUP_ROBOT_EVENT
    GROUP_ADD_ROBOT = "GROUP_ADD_ROBOT"
    GROUP_DEL_ROBOT = "GROUP_DEL_ROBOT"
    GROUP_MSG_REJECT = "GROUP_MSG_REJECT"
    GROUP_MSG_RECEIVE = "GROUP_MSG_RECEIVE"


class Event(BaseEvent):
    __type__: EventType

    # event id from payload id
    event_id: Optional[str] = None

    @override
    def get_event_name(self) -> str:
        return self.__type__

    @override
    def get_event_description(self) -> str:
        return escape_tag(str(self.dict()))

    @override
    def get_message(self) -> Message:
        raise ValueError("Event has no message!")

    @override
    def get_user_id(self) -> str:
        raise ValueError("Event has no context!")

    @override
    def get_session_id(self) -> str:
        raise ValueError("Event has no context!")

    @override
    def is_tome(self) -> bool:
        return False


EVENT_CLASSES: dict[str, type[Event]] = {}


def register_event_class(event_class: type[E]) -> type[E]:
    EVENT_CLASSES[event_class.__type__.value] = event_class
    return event_class


# Meta Event
class MetaEvent(Event):
    @override
    def get_type(self) -> str:
        return "meta_event"


@register_event_class
class ReadyEvent(MetaEvent):
    __type__ = EventType.READY
    version: int
    session_id: str
    user: User
    shard: tuple[int, int]


@register_event_class
class ResumedEvent(MetaEvent):
    __type__ = EventType.RESUMED


# Notice Event
class NoticeEvent(Event):
    @override
    def get_type(self) -> str:
        return "notice"


# Guild Event
class GuildEvent(NoticeEvent, Guild):
    op_user_id: str


@register_event_class
class GuildCreateEvent(GuildEvent):
    __type__ = EventType.GUILD_CREATE


@register_event_class
class GuildUpdateEvent(GuildEvent):
    __type__ = EventType.GUILD_UPDATE


@register_event_class
class GuildDeleteEvent(GuildEvent):
    __type__ = EventType.GUILD_DELETE


# Channel Event
class ChannelEvent(NoticeEvent, Channel):
    op_user_id: str


@register_event_class
class ChannelCreateEvent(ChannelEvent):
    __type__ = EventType.CHANNEL_CREATE


@register_event_class
class ChannelUpdateEvent(ChannelEvent):
    __type__ = EventType.CHANNEL_UPDATE


@register_event_class
class ChannelDeleteEvent(ChannelEvent):
    __type__ = EventType.CHANNEL_DELETE


# Guild Member Event
class GuildMemberEvent(NoticeEvent, Member):
    guild_id: str
    op_user_id: str

    @override
    def get_user_id(self) -> str:
        return self.user.id  # type: ignore

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            f"Notice {getattr(self.user, 'username', None)}"
            f"@[Guild:{self.guild_id}] Roles:{self.roles}"
        )

    @override
    def get_session_id(self) -> str:
        return f"guild_{self.guild_id}_{self.user.id}"  # type: ignore


@register_event_class
class GuildMemberAddEvent(GuildMemberEvent):
    __type__ = EventType.GUILD_MEMBER_ADD


@register_event_class
class GuildMemberUpdateEvent(GuildMemberEvent):
    __type__ = EventType.GUILD_MEMBER_UPDATE


@register_event_class
class GuildMemberRemoveEvent(GuildMemberEvent):
    __type__ = EventType.GUILD_MEMBER_REMOVE


# Message Event
class MessageEvent(Event):
    to_me: bool = False

    @override
    def get_type(self) -> str:
        return "message"

    @override
    def is_tome(self) -> bool:
        return self.to_me


class GuildMessageEvent(MessageEvent, GuildMessage):
    reply: Optional[GuildMessage] = None
    """
    :说明: 消息中提取的回复消息，内容为 ``get_message_of_id`` API 返回结果

    :类型: ``Optional[GuildMessage]``
    """

    @override
    def get_user_id(self) -> str:
        return self.author.id

    @override
    def get_session_id(self) -> str:
        return f"guild_{self.guild_id}_channel_{self.channel_id}_{self.author.id}"

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            f"Message {self.id} from "
            f"{getattr(self.author, 'username', None)}"
            f"@[Guild:{self.guild_id}/{self.channel_id}] "
            f"Roles:{getattr(self.member, 'roles', None)}: {self.get_message()!r}"
        )

    @override
    def get_message(self) -> Message:
        if not hasattr(self, "_message"):
            setattr(self, "_message", Message.from_guild_message(self))
        return getattr(self, "_message")


@register_event_class
class MessageCreateEvent(GuildMessageEvent):
    __type__ = EventType.MESSAGE_CREATE


@register_event_class
class MessageDeleteEvent(NoticeEvent, MessageDelete):
    __type__ = EventType.MESSAGE_DELETE

    @override
    def get_session_id(self) -> str:
        return (
            f"guild_{self.message.guild_id}_"
            f"channel_{self.message.channel_id}_{self.op_user.id}"
        )


@register_event_class
class AtMessageCreateEvent(GuildMessageEvent):
    __type__ = EventType.AT_MESSAGE_CREATE

    to_me: bool = True


@register_event_class
class PublicMessageDeleteEvent(MessageDeleteEvent):
    __type__ = EventType.PUBLIC_MESSAGE_DELETE


@register_event_class
class DirectMessageCreateEvent(GuildMessageEvent):
    __type__ = EventType.DIRECT_MESSAGE_CREATE

    to_me: bool = True

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            f"Message {self.id} from "
            f"{getattr(self.author, 'username', None)}: {self.get_message()!r}"
        )


@register_event_class
class DirectMessageDeleteEvent(MessageDeleteEvent):
    __type__ = EventType.DIRECT_MESSAGE_DELETE


class QQMessageEvent(MessageEvent, QQMessage):
    _reply_seq: int = -1

    @override
    def get_message(self) -> Message:
        if not hasattr(self, "_message"):
            setattr(self, "_message", Message.from_qq_message(self))
        return getattr(self, "_message")


@register_event_class
class C2CMessageCreateEvent(QQMessageEvent):
    __type__ = EventType.C2C_MESSAGE_CREATE

    author: FriendAuthor
    to_me: bool = True

    @override
    def get_user_id(self) -> str:
        return self.author.user_openid

    @override
    def get_session_id(self) -> str:
        return f"friend_{self.author.user_openid}"

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            f"Message {self.id} from {self.author.id}: {self.get_message()!r}"
        )


@register_event_class
class GroupAtMessageCreateEvent(QQMessageEvent):
    __type__ = EventType.GROUP_AT_MESSAGE_CREATE

    author: GroupMemberAuthor
    group_openid: str
    to_me: bool = True

    @override
    def get_message(self) -> Message:
        # tmp fix to remove space before text due to at not in content
        msg = Message.from_qq_message(self)
        if msg and msg[0].type == "text":
            msg[0].data["text"] = msg[0].data["text"].lstrip()
        if not hasattr(self, "_message"):
            setattr(self, "_message", msg)
        return getattr(self, "_message")

    @override
    def get_user_id(self) -> str:
        return self.author.member_openid

    @override
    def get_session_id(self) -> str:
        return f"group_{self.group_openid}_{self.author.member_openid}"

    @override
    def get_event_description(self) -> str:
        return escape_tag(
            f"Message {self.id} from "
            f"{self.author.member_openid}@[Group:{self.group_openid}]: "
            f"{self.get_message()!r}"
        )


@register_event_class
class InteractionCreateEvent(NoticeEvent, ButtonInteraction):
    __type__ = EventType.INTERACTION_CREATE

    @override
    def get_user_id(self) -> str:
        if self.chat_type == 0:
            return cast(str, self.data.resolved.user_id)
        elif self.chat_type == 1:
            return cast(str, self.group_member_openid)
        elif self.chat_type == 2:
            return cast(str, self.user_openid)
        raise ValueError(f"Unknown chat_type: {self.chat_type}")

    @override
    def get_session_id(self) -> str:
        if self.chat_type == 0:
            return (
                f"guild_{self.guild_id}_channel_{self.channel_id}"
                f"_{self.data.resolved.user_id}"
            )
        elif self.chat_type == 1:
            return f"group_{self.group_openid}_{self.group_member_openid}"
        elif self.chat_type == 2:
            return f"friend_{self.user_openid}"
        raise ValueError(f"Unknown chat_type: {self.chat_type}")


# Message Audit Event
class MessageAuditEvent(NoticeEvent, MessageAudited): ...


@register_event_class
class MessageAuditPassEvent(MessageAuditEvent):
    __type__ = EventType.MESSAGE_AUDIT_PASS


@register_event_class
class MessageAuditRejectEvent(MessageAuditEvent):
    __type__ = EventType.MESSAGE_AUDIT_REJECT


# Message Reaction Event
class MessageReactionEvent(NoticeEvent, MessageReaction):
    @override
    def get_user_id(self) -> str:
        return self.user_id

    @override
    def get_session_id(self) -> str:
        return f"guild_{self.guild_id}_channel_{self.channel_id}_{self.user_id}"


@register_event_class
class MessageReactionAddEvent(MessageReactionEvent):
    __type__ = EventType.MESSAGE_REACTION_ADD


@register_event_class
class MessageReactionRemoveEvent(MessageReactionEvent):
    __type__ = EventType.MESSAGE_REACTION_REMOVE


# Audio Event
class AudioEvent(NoticeEvent, AudioAction): ...


@register_event_class
class AudioStartEvent(AudioEvent):
    __type__ = EventType.AUDIO_START


@register_event_class
class AudioFinishEvent(AudioEvent):
    __type__ = EventType.AUDIO_FINISH


@register_event_class
class AudioOnMicEvent(AudioEvent):
    __type__ = EventType.AUDIO_ON_MIC


@register_event_class
class AudioOffMicEvent(AudioEvent):
    __type__ = EventType.AUDIO_OFF_MIC


# Forum Event
class ForumEvent(NoticeEvent, ForumSourceInfo):
    @override
    def get_user_id(self) -> str:
        return self.author_id

    @override
    def get_session_id(self) -> str:
        return f"guild_{self.guild_id}_channel_{self.channel_id}_{self.author_id}"


class ForumThreadEvent(ForumEvent, Thread[RichText]): ...


@register_event_class
class ForumThreadCreateEvent(ForumThreadEvent):
    __type__ = EventType.FORUM_THREAD_CREATE


@register_event_class
class ForumThreadUpdateEvent(ForumThreadEvent):
    __type__ = EventType.FORUM_THREAD_UPDATE


@register_event_class
class ForumThreadDeleteEvent(ForumThreadEvent):
    __type__ = EventType.FORUM_THREAD_DELETE


class ForumPostEvent(ForumEvent, Post): ...


@register_event_class
class ForumPostCreateEvent(ForumPostEvent):
    __type__ = EventType.FORUM_POST_CREATE


@register_event_class
class ForumPostDeleteEvent(ForumPostEvent):
    __type__ = EventType.FORUM_POST_DELETE


class ForumReplyEvent(ForumEvent, Reply): ...


@register_event_class
class ForumReplyCreateEvent(ForumReplyEvent):
    __type__ = EventType.FORUM_REPLY_CREATE


@register_event_class
class ForumReplyDeleteEvent(ForumReplyEvent):
    __type__ = EventType.FORUM_REPLY_DELETE


@register_event_class
class ForumPublishAuditResult(ForumEvent, ForumAuditResult):
    __type__ = EventType.FORUM_PUBLISH_AUDIT_RESULT


class OpenForumEvent(NoticeEvent, ForumSourceInfo):
    @override
    def get_user_id(self) -> str:
        return self.author_id

    @override
    def get_session_id(self) -> str:
        return f"guild_{self.guild_id}_channel_{self.channel_id}_{self.author_id}"


@register_event_class
class OpenForumThreadCreateEvent(OpenForumEvent):
    __type__ = EventType.OPEN_FORUM_THREAD_CREATE


@register_event_class
class OpenForumThreadUpdateEvent(OpenForumEvent):
    __type__ = EventType.OPEN_FORUM_THREAD_UPDATE


@register_event_class
class OpenForumThreadDeleteEvent(OpenForumEvent):
    __type__ = EventType.OPEN_FORUM_THREAD_DELETE


@register_event_class
class OpenForumPostCreateEvent(OpenForumEvent):
    __type__ = EventType.OPEN_FORUM_POST_CREATE


@register_event_class
class OpenForumPostDeleteEvent(OpenForumEvent):
    __type__ = EventType.OPEN_FORUM_POST_DELETE


@register_event_class
class OpenForumReplyCreateEvent(OpenForumEvent):
    __type__ = EventType.OPEN_FORUM_REPLY_CREATE


@register_event_class
class OpenForumReplyDeleteEvent(OpenForumEvent):
    __type__ = EventType.OPEN_FORUM_REPLY_DELETE


# Friend Robot Event
class FriendRobotEvent(NoticeEvent):
    timestamp: datetime
    openid: str

    @override
    def get_user_id(self) -> str:
        return self.openid

    @override
    def get_session_id(self) -> str:
        return f"friend_{self.openid}"


@register_event_class
class FriendAddEvent(FriendRobotEvent):
    __type__ = EventType.FRIEND_ADD


@register_event_class
class FriendDelEvent(FriendRobotEvent):
    __type__ = EventType.FRIEND_DEL


@register_event_class
class C2CMsgRejectEvent(FriendRobotEvent):
    __type__ = EventType.C2C_MSG_REJECT


@register_event_class
class C2CMsgReceiveEvent(FriendRobotEvent):
    __type__ = EventType.C2C_MSG_RECEIVE


# Group Robot Event
class GroupRobotEvent(NoticeEvent):
    timestamp: datetime
    group_openid: str
    op_member_openid: str

    @override
    def get_user_id(self) -> str:
        return self.op_member_openid

    @override
    def get_session_id(self) -> str:
        return f"group_{self.group_openid}_{self.op_member_openid}"


@register_event_class
class GroupAddRobotEvent(GroupRobotEvent):
    __type__ = EventType.GROUP_ADD_ROBOT


@register_event_class
class GroupDelRobotEvent(GroupRobotEvent):
    __type__ = EventType.GROUP_DEL_ROBOT


@register_event_class
class GroupMsgRejectEvent(GroupRobotEvent):
    __type__ = EventType.GROUP_MSG_REJECT


@register_event_class
class GroupMsgReceiveEvent(GroupRobotEvent):
    __type__ = EventType.GROUP_MSG_RECEIVE


__all__ = [
    "EVENT_CLASSES",
    "EventType",
    "Event",
    "MetaEvent",
    "ReadyEvent",
    "ResumedEvent",
    "NoticeEvent",
    "GuildEvent",
    "GuildCreateEvent",
    "GuildUpdateEvent",
    "GuildDeleteEvent",
    "ChannelEvent",
    "ChannelCreateEvent",
    "ChannelUpdateEvent",
    "ChannelDeleteEvent",
    "GuildMemberEvent",
    "GuildMemberAddEvent",
    "GuildMemberUpdateEvent",
    "GuildMemberRemoveEvent",
    "MessageEvent",
    "GuildMessageEvent",
    "MessageCreateEvent",
    "MessageDeleteEvent",
    "AtMessageCreateEvent",
    "PublicMessageDeleteEvent",
    "DirectMessageCreateEvent",
    "DirectMessageDeleteEvent",
    "QQMessageEvent",
    "C2CMessageCreateEvent",
    "GroupAtMessageCreateEvent",
    "InteractionCreateEvent",
    "MessageAuditEvent",
    "MessageAuditPassEvent",
    "MessageAuditRejectEvent",
    "MessageReactionEvent",
    "MessageReactionAddEvent",
    "MessageReactionRemoveEvent",
    "AudioEvent",
    "AudioStartEvent",
    "AudioFinishEvent",
    "AudioOnMicEvent",
    "AudioOffMicEvent",
    "ForumEvent",
    "ForumThreadEvent",
    "ForumThreadCreateEvent",
    "ForumThreadUpdateEvent",
    "ForumThreadDeleteEvent",
    "ForumPostEvent",
    "ForumPostCreateEvent",
    "ForumPostDeleteEvent",
    "ForumReplyEvent",
    "ForumReplyCreateEvent",
    "ForumReplyDeleteEvent",
    "ForumPublishAuditResult",
    "OpenForumEvent",
    "OpenForumThreadCreateEvent",
    "OpenForumThreadUpdateEvent",
    "OpenForumThreadDeleteEvent",
    "OpenForumPostCreateEvent",
    "OpenForumPostDeleteEvent",
    "OpenForumReplyCreateEvent",
    "OpenForumReplyDeleteEvent",
    "FriendRobotEvent",
    "FriendAddEvent",
    "FriendDelEvent",
    "C2CMsgRejectEvent",
    "C2CMsgReceiveEvent",
    "GroupRobotEvent",
    "GroupAddRobotEvent",
    "GroupDelRobotEvent",
    "GroupMsgRejectEvent",
    "GroupMsgReceiveEvent",
]



from typing import Literal, overload

from nonebot.compat import PYDANTIC_V2

__all__ = ("model_validator", "field_validator")

if PYDANTIC_V2:
    from pydantic import field_validator as field_validator
    from pydantic import model_validator as model_validator
else:
    from pydantic import validator, root_validator

    @overload
    def model_validator(*, mode: Literal["before"]): ...

    @overload
    def model_validator(*, mode: Literal["after"]): ...

    def model_validator(*, mode: Literal["before", "after"]):
        return root_validator(pre=mode == "before", allow_reuse=True)

    def field_validator(__field, *fields, mode: Literal["before", "after"] = "after"):
        return validator(__field, *fields, pre=mode == "before", allow_reuse=True)



from typing import Union

from nonebot.permission import Permission

from .event import MessageCreateEvent, AtMessageCreateEvent


async def _guild_channel_admin(
    event: Union[AtMessageCreateEvent, MessageCreateEvent]
) -> bool:
    return "5" in getattr(event.member, "roles", ())


async def _guild_admin(event: Union[AtMessageCreateEvent, MessageCreateEvent]) -> bool:
    return "2" in getattr(event.member, "roles", ())


async def _guild_owner(event: Union[AtMessageCreateEvent, MessageCreateEvent]) -> bool:
    return "4" in getattr(event.member, "roles", ())


GUILD_CHANNEL_ADMIN: Permission = Permission(_guild_channel_admin)
"""匹配任意子频道管理员聊消息类型事件"""
GUILD_ADMIN: Permission = Permission(_guild_admin)
"""匹配任意频道管理员聊消息类型事件"""
GUILD_OWNER: Permission = Permission(_guild_owner)
"""匹配任意频道群主群消息类型事件"""

__all__ = [
    "GUILD_CHANNEL_ADMIN",
    "GUILD_ADMIN",
    "GUILD_OWNER",
]



import asyncio
from typing import TYPE_CHECKING, Optional

if TYPE_CHECKING:
    from .event import MessageAuditEvent


class AuditResultStore:
    def __init__(self) -> None:
        self._futures: dict[str, asyncio.Future] = {}

    def add_result(self, result: "MessageAuditEvent"):
        audit_id = result.audit_id
        if not audit_id:
            raise ValueError("audit_id cannot be empty")
        if future := self._futures.get(audit_id):
            future.set_result(result)

    async def fetch(
        self, audit_id: str, timeout: Optional[float] = None
    ) -> "MessageAuditEvent":
        future = asyncio.get_event_loop().create_future()
        self._futures[audit_id] = future
        try:
            return await asyncio.wait_for(future, timeout)
        finally:
            del self._futures[audit_id]


audit_result = AuditResultStore()



from typing import Optional
from datetime import datetime
from urllib.parse import urlparse

from pydantic import BaseModel

from nonebot.adapters.qq.compat import field_validator


class FriendAuthor(BaseModel):
    id: str
    user_openid: str


class GroupMemberAuthor(BaseModel):
    id: str
    member_openid: str


class Attachment(BaseModel):
    content_type: str
    filename: Optional[str] = None
    height: Optional[int] = None
    width: Optional[int] = None
    size: Optional[int] = None
    url: Optional[str] = None

    @field_validator("url", mode="after")
    def check_url(cls, v: str):
        if v and not urlparse(v).hostname:
            return f"https://{v}"
        return v


class Media(BaseModel):
    file_info: str


class QQMessage(BaseModel):
    id: str
    content: str
    timestamp: str
    attachments: Optional[list[Attachment]] = None


class PostC2CMessagesReturn(BaseModel):
    id: Optional[str] = None
    timestamp: Optional[datetime] = None


class PostGroupMessagesReturn(BaseModel):
    id: Optional[str] = None
    timestamp: Optional[datetime] = None


class PostC2CFilesReturn(BaseModel):
    file_uuid: Optional[str] = None
    file_info: Optional[str] = None
    ttl: Optional[int] = None


class PostGroupFilesReturn(BaseModel):
    file_uuid: Optional[str] = None
    file_info: Optional[str] = None
    ttl: Optional[int] = None


class GroupMember(BaseModel):
    member_openid: str
    join_timestamp: datetime


class PostGroupMembersReturn(BaseModel):
    members: list[GroupMember]
    next_index: Optional[int] = None


__all__ = [
    "FriendAuthor",
    "GroupMemberAuthor",
    "Attachment",
    "Media",
    "QQMessage",
    "PostC2CMessagesReturn",
    "PostGroupMessagesReturn",
    "PostC2CFilesReturn",
    "GroupMember",
    "PostGroupMembersReturn",
    "PostGroupFilesReturn",
]



from .qq import *  # noqa: F403
from .guild import *  # noqa: F403
from .common import *  # noqa: F403
from .payload import Hello as Hello
from .payload import Opcode as Opcode
from .payload import Resume as Resume
from .payload import Payload as Payload
from .payload import Dispatch as Dispatch
from .payload import Identify as Identify
from .payload import Heartbeat as Heartbeat
from .payload import Reconnect as Reconnect
from .payload import PayloadType as PayloadType
from .payload import HeartbeatAck as HeartbeatAck
from .payload import InvalidSession as InvalidSession



from functools import partial
from collections.abc import Awaitable
from typing_extensions import ParamSpec, Concatenate
from typing import TYPE_CHECKING, Any, Generic, TypeVar, Callable, Optional, overload

from nonebot.utils import logger_wrapper

if TYPE_CHECKING:
    from .bot import Bot

B = TypeVar("B", bound="Bot")
R = TypeVar("R")
P = ParamSpec("P")

log = logger_wrapper("QQ")


def escape(s: str) -> str:
    return s.replace("&", "&amp;").replace("<", "&lt;").replace(">", "&gt;")


def unescape(s: str) -> str:
    return s.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")


def exclude_none(data: dict[str, Any]) -> dict[str, Any]:
    return {k: v for k, v in data.items() if v is not None}


class API(Generic[B, P, R]):
    def __init__(self, func: Callable[Concatenate[B, P], Awaitable[R]]) -> None:
        self.func = func

    def __set_name__(self, owner: type[B], name: str) -> None:
        self.name = name

    @overload
    def __get__(self, obj: None, objtype: type[B]) -> "API[B, P, R]": ...

    @overload
    def __get__(
        self, obj: B, objtype: Optional[type[B]]
    ) -> Callable[P, Awaitable[R]]: ...

    def __get__(
        self, obj: Optional[B], objtype: Optional[type[B]] = None
    ) -> "API[B, P, R] | Callable[P, Awaitable[R]]":
        if obj is None:
            return self

        return partial(obj.call_api, self.name)  # type: ignore

    async def __call__(self, inst: B, *args: P.args, **kwds: P.kwargs) -> R:
        return await self.func(inst, *args, **kwds)



from .bot import Bot as Bot
from .utils import log as log
from .event import *  # noqa: F403
from .utils import escape as escape
from .adapter import Adapter as Adapter
from .message import Message as Message
from .permission import *  # noqa: F403
from .utils import unescape as unescape
from .exception import ActionFailed as ActionFailed
from .exception import NetworkError as NetworkError
from .message import MessageSegment as MessageSegment
from .exception import AuditException as AuditException
from .exception import NoLogException as NoLogException
from .exception import ApiNotAvailable as ApiNotAvailable
from .exception import QQAdapterException as QQAdapterException
from .exception import RateLimitException as RateLimitException
from .exception import UnauthorizedException as UnauthorizedException
